# Volatility EditBox plugin
#
# Author: Bridgey the Geek <bridgeythegeek@gmail.com>
#
# This plugin is free software; you can redistribute it and/or modify
# it under the terms of GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This plugin is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PRACTICAL PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this plugin. If not, see <http://www.gnu.org/licenses/>.
#
# This work heavily inspired by  GDI Utilities from Dr Brendan Dolan-Gavitt PhD.
# <http://www.cc.gatech.edu/~brendan/volatility/>
#
# The iteration of the Windows objects is borrowed from the Windows plugin.
# <https://github.com/volatilityfoundation/volatility/blob/master/volatility/plugins/gui/windows.py>
#
# This plugin wouldn't exist without the assistance of those on the volusers
# mailing list <http://lists.volatilesystems.com/mailman/listinfo/vol-users>.

"""
@author     : Bridgey the Geek
@license    : GPL 2 or later
@contact    : bridgeythegeek@gmail.com
"""

import os
import struct
import hashlib

import volatility
import volatility.debug as debug
import volatility.obj as obj
import volatility.plugins.common as common
import volatility.plugins.gui.messagehooks as messagehooks
import volatility.utils as utils
import volatility.win32 as win32

# Add a structure to the x86 GDI types.
gdi_types_x86 = {
    '_EDIT_x86': [ 0xF6, {
        'hBuf': [ 0x00, ['pointer', ['pointer', ['unsigned long']]]],
        'hWnd': [ 0x38, ['unsigned long']],
        'parenthWnd': [ 0x58, ['unsigned long']],
        'nChars': [0x0C, ['unsigned long']],
        'selStart': [0x14, ['unsigned long']],
        'selEnd': [0x18, ['unsigned long']],
        'pwdChar': [0x30, ['unsigned short']],
        'bEncKeyXP': [0xEC, ['unsigned char']], # XP
        'bEncKey': [0xF4, ['unsigned char']] # Win7/2008R2
        # TODO: bEncKey is hacky. Should have diff types for each.
    }],
    '_LISTBOX_x86': [ 0x40, {
        'hWnd': [0x00, ['unsigned long']],
        'parenthWnd': [0x04, ['unsigned long']],
        'atomHandle': [0x08, ['unsigned long']],
        'firstVisibleRow': [0x10, ['unsigned long']],
        'caretPos': [0x14, ['long']],
        'rowsVisible': [0x1C, ['unsigned long']],
        'itemCount': [0x20, ['unsigned long']],
        'stringsStart': [0x2C, ['unsigned long']],
        'stringsLength': [0x34, ['unsigned long']]
    }],
    '_COMBOBOX_x86': [0x28, {
        'hWnd': [0x00, ['unsigned long']],
        'parenthWnd': [0x04, ['unsigned long']],
        # 'atomHandle': [0x08, ['unsigned long']],
        # 'edithWnd': [0x4C, ['unsigned long']],
        'combolboxhWnd': [0x50, ['unsigned long']]
    }]
}

# Add a structure to the x64 GDI types.
gdi_types_x64 = {
    '_EDIT_x64': [ 0x142, {
        'hBuf': [0x00, ['pointer', ['pointer', ['unsigned long']]]],
        'hWnd': [0x40, ['unsigned long']],
        'parenthWnd': [0x60, ['unsigned long']],
        'nChars': [0x10, ['unsigned long']],
        'selStart': [0x18, ['unsigned long']],
        'selEnd': [0x20, ['unsigned long']],
        'pwdChar': [0x34, ['unsigned short']],
        'bEncKey': [0x140, ['unsigned char']]
    } ],
    '_LISTBOX_x64': [ 0x100, {
        'hWnd': [0x00, ['unsigned long']],
        'parenthWnd': [0x08, ['unsigned long']],
        'firstVisibleRow': [0x20, ['unsigned long']],
        'caretPos': [0x28, ['unsigned long']],
        'rowsVisible': [0x2C, ['unsigned long']],
        'itemCount': [0x30, ['unsigned long']],
        'stringsStart': [0x40, ['unsigned long']],
        'stringsLength': [0x4C, ['unsigned long']]
    } ],
    '_COMBOBOX_x64': [0x68, {
        'hWnd': [0x00, ['unsigned long']],
        'parenthWnd': [0x08, ['unsigned long']],
        'combolboxhWnd': [0x60, ['unsigned long']]
    }]
}

# Define the _EDIT_BOX_x86 structure.
class _EDIT_x86(obj.CType):

    def get_hBuf(self):
        # Double dereference
        ptr = (struct.unpack('<l', self.obj_vm.read(self.hBuf.obj_offset, 4)))[0]
        if not self.obj_vm.is_valid_address(ptr): return None
        ptr = (struct.unpack('<l', self.obj_vm.read(ptr, 4)))[0]
        if not self.obj_vm.is_valid_address(ptr): return None
        return ptr

    def get_text(self):
        key = self.bEncKeyXP if True else self.bEncKey
        s = self.obj_vm.read(self.get_hBuf(), self.nChars * 2)
        if not self.pwdChar == 0x00:
            s = EditBox.RtlRunDecodeUnicodeString(key, s)
        return '' if not s else s

# Define the _EDIT_BOX_x64 structure.
class _EDIT_x64(obj.CType):

    def get_hBuf(self):
        # Double dereference
        ptr = (struct.unpack('<l', self.obj_vm.read(self.hBuf.obj_offset, 4)))[0]
        if not self.obj_vm.is_valid_address(ptr): return None
        ptr = (struct.unpack('<l', self.obj_vm.read(ptr, 4)))[0]
        if not self.obj_vm.is_valid_address(ptr): return None
        return ptr

    def get_text(self):
        s = self.obj_vm.read(self.get_hBuf(), self.nChars * 2)
        if not self.pwdChar == 0x00:
            s = EditBox.RtlRunDecodeUnicodeString(self.bEncKey, s)
        return '' if not s else s

# Define the _LISTBOX_x86 structure.
class _LISTBOX_x86(obj.CType):

    def get_text(self):
        string_array = self.obj_vm.read(self.stringsStart, self.stringsLength).split(b'\x00\x00')[:-1]
        return ', '.join(string_array) if string_array else ''

# Define the _LISTBOX_x64 structure.
class _LISTBOX_x64(obj.CType):

    def get_text(self):
        string_array = self.obj_vm.read(self.stringsStart, self.stringsLength).split(b'\x00\x00')[:-1]
        return ', '.join(string_array) if string_array else ''

# Define the _COMBOBOX_x86 structure.
class _COMBOBOX_x86(obj.CType):
    pass

# Define the _COMBOBOX_x64 structure.
class _COMBOBOX_x64(obj.CType):
    pass

class EditBoxVTypes(obj.ProfileModification):
    """This modification adds the gdi_types_x(86|64)."""

    def check(self, profile):
        m = profile.metadata
        return m.get('os', None) == 'windows'

    def modification(self, profile):
        profile.vtypes.update(gdi_types_x86)
        profile.vtypes.update(gdi_types_x64)

class EditBoxObjectClasses(obj.ProfileModification):
    """Add the new class definitions."""

    def modification(self, profile):
        profile.object_classes.update({
            '_EDIT_x86': _EDIT_x86,
            '_EDIT_x64': _EDIT_x64,
            '_LISTBOX_x86': _LISTBOX_x86,
            '_LISTBOX_x64': _LISTBOX_x64,
            '_COMBOBOX_x86': _COMBOBOX_x86,
            '_COMBOBOX_x64': _COMBOBOX_x64
        })

class EditBox(messagehooks.MessageHooks):
    """Dumps various data from ComCtl Edit controls (experimental: ListBox, ComboBox)"""

    def __init__(self, config, *args, **kwargs):
        common.AbstractWindowsCommand.__init__(self, config, *args, **kwargs)
        # Plugin parameters
        self._config.add_option('DUMP-DIR', short_option='D', default=None,
                                help='Directory to which to dump found text',
                                action='store', type='str')
        self._config.add_option('NULLS', short_option='n', default=False,
                                help='keep nulls when outputting to stdout', action='store_true')
        self._config.add_option('PID', short_option='p', default=None,
                                help='This Process ID', action='store', type='int')
        self._config.add_option('MINIMAL', short_option='m', default=None,
                                help='Process name and text only',
                                action='store_true')
        self._config.add_option('EXPERIMENTAL', short_option='e', default=None,
                                help='Enable experimental options',
                                action='store_true')
        self._config.add_option('EXPERIMENTAL-ONLY', short_option='E', default=None,
                                help='Only do experimental options',
                                action='store_true')
        # Object variables
        self._addr_space = utils.load_as(self._config)
        self._profile_is_32bit = (self._addr_space.profile.metadata['memory_model'] == '32bit')
        self._tagWND_size = self._addr_space.profile.get_obj_size('tagWND')

    @staticmethod
    def is_valid_profile(profile):
        """Returns True if the plugin is valid for the current profile"""

        return profile.metadata.get('os', 'unknown') == 'windows'

    @staticmethod
    def RtlRunDecodeUnicodeString(key, data):
        s = ''.join([chr(ord(data[i-1]) ^ ord(data[i]) ^ key) for i in range(1,len(data))])
        s = chr(ord(data[0]) ^ (key | 0x43)) + s
        return s

    def dump_edit(self, dump_to_file, rm_nulls, atom_class_name, context, outfd, wnd, task):
        """Process an Edit"""

        task_space = task.get_process_address_space()

        addr_wndextra = wnd.v() + self._tagWND_size
        val = wnd.obj_vm.read(addr_wndextra, wnd.cbwndExtra)

        # Build an _EDIT_BOX_x?? object from the WndExtra bytes
        if self._profile_is_32bit or task.IsWow64:
            edit_box_type = '_EDIT_x86'
            wndextra = struct.unpack('<I', val)[0]
        else:
            edit_box_type = '_EDIT_x64'
            wndextra = struct.unpack('<Q', val)[0]
        
        editbox = obj.Object(edit_box_type, offset=wndextra, vm=task_space)
        valid_hbuf = not (editbox.get_hBuf() is None)

        # wndextra was 0 or some other invalid address
        if editbox is None:
            return

        isPassword = not (editbox.pwdChar == 0x00)
        
        if valid_hbuf:
            # Get the text from the control
            the_text = editbox.get_text()
            # Calc the MD5
            md5 = hashlib.md5(the_text).hexdigest()

        # Dump the info
        if self._config.minimal:
            outfd.write('{0}:{1}\n'.format(wnd.Process.UniqueProcessId, wnd.Process.ImageFileName))
        else:
            outfd.write('{0}\n'.format('*' * 55))

            if not valid_hbuf:
                outfd.write('{0} invalid hBuf - can\'t get text {0}\n'.format('*' * 12))
                outfd.write('{0}\n'.format('*' * 55))

            outfd.write('Wnd context          : {0}\n'.format(context))
            outfd.write('pointer-to tagWND    : {0:#x} [{1:#x}]\n'.format(
                wnd.obj_offset, task_space.vtop(wnd.obj_offset)))
            outfd.write('pid                  : {0}\n'.format(wnd.Process.UniqueProcessId))
            outfd.write('imageFileName        : {0}\n'.format(wnd.Process.ImageFileName))

            if not self._profile_is_32bit:
                outfd.write('wow64                : {0}\n'.format('Yes' if task.IsWow64 else 'No'))

            outfd.write('atom_class           : {0}\n'.format(atom_class_name))
            outfd.write('address-of cbwndExtra: {0:#x} [{1:#x}]\n'.format(
                wnd.cbwndExtra.obj_offset, task_space.vtop(wnd.cbwndExtra.obj_offset)))
            outfd.write('value-of cbwndExtra  : {0} ({0:#x})\n'.format(wnd.cbwndExtra))
            outfd.write('address-of WndExtra  : {0:#x} [{1:#x}]\n'.format(
                addr_wndextra, task_space.vtop(addr_wndextra)))
            outfd.write('value-of WndExtra    : {0:#x} [{1:#x}]\n'.format(wndextra, task_space.vtop(wndextra)))

            if valid_hbuf:
                outfd.write('pointer-to hBuf      : {0:#x} [{1:#x}]\n'.format(
                    editbox.get_hBuf(), task_space.vtop(editbox.get_hBuf())))
            else:
                outfd.write('pointer-to hBuf      : -invalid-\n')

            outfd.write('hWnd                 : {0:#x}\n'.format(editbox.hWnd))
            outfd.write('parenthWnd           : {0:#x}\n'.format(editbox.parenthWnd))
            outfd.write('nChars               : {0} ({0:#x})\n'.format(editbox.nChars))
            outfd.write('selStart             : {0} ({0:#x})\n'.format(editbox.selStart))
            outfd.write('selEnd               : {0} ({0:#x})\n'.format(editbox.selEnd))

            if valid_hbuf:
                outfd.write('text_md5             : {0}\n'.format(md5))
        
            outfd.write('isPwdControl         : {0}\n'.format('Yes' if isPassword else 'No'))
            if isPassword:
                outfd.write('pwdChar              : {0:#x}\n'.format(editbox.pwdChar))
        
        if valid_hbuf:
            if dump_to_file:  # Write to file, named as per the MD5
                with open(os.path.join(self._config.dump_dir, md5 + '.txt'), 'wb') as f:
                    f.write(the_text)
            else:  # Write to the screen
                if rm_nulls:
                    the_text = the_text.replace('\x00', '')
                outfd.write('{0}\n'.format(the_text))

    def dump_listbox(self, dump_to_file, atom_class_name, context, outfd, wnd, task):
        """Process a ListBox"""
        task_space = task.get_process_address_space()

        addr_wndextra = wnd.v() + self._tagWND_size
        val = wnd.obj_vm.read(addr_wndextra, wnd.cbwndExtra)

        # Build a _LISTBOX_x?? object from the WndExtra bytes
        if self._profile_is_32bit:
            listbox_type = '_LISTBOX_x86'
            wndextra = struct.unpack('<I', val)[0]
        elif task.IsWow64:
            listbox_type = '_LISTBOX_x86'
            wndextra = struct.unpack('<I', val)[0]
        else:
            listbox_type = '_LISTBOX_x64'
            wndextra = struct.unpack('<Q', val)[0]
        listbox = obj.Object(listbox_type, offset=wndextra, vm=task_space)

        outfd.write('{0}\n'.format('*' * 55))
        outfd.write('*** Experimental {0}\n'.format('*' * 38))
        outfd.write('{0}\n'.format('*' * 55))
        outfd.write('Wnd context          : {0}\n'.format(context))
        outfd.write('pointer-to tagWND    : {0:#x} [{1:#x}]\n'.format(wnd.obj_offset, task_space.vtop(wnd.obj_offset)))
        outfd.write('pid                  : {0}\n'.format(wnd.Process.UniqueProcessId))
        outfd.write('process              : {0}\n'.format(wnd.Process.ImageFileName))
        if not self._profile_is_32bit:
            outfd.write('wow64                : {0}\n'.format('Yes' if task.IsWow64 else 'No'))
        outfd.write('atom_class           : {0}\n'.format(atom_class_name))
        outfd.write('address-of cbwndExtra: {0:#x} [{1:#x}]\n'.format(
            wnd.cbwndExtra.obj_offset, task_space.vtop(wnd.cbwndExtra.obj_offset)))
        outfd.write('value-of cbwndExtra  : {0} ({0:#x})\n'.format(wnd.cbwndExtra))
        outfd.write('address-of WndExtra  : {0:#x} [{1:#x}]\n'.format(addr_wndextra, task_space.vtop(addr_wndextra)))
        outfd.write('value-of WndExtra    : {0:#x} [{1:#x}]\n'.format(wndextra, task_space.vtop(wndextra)))
        outfd.write('firstVisibleRow      : {0} ({0:#x})\n'.format(listbox.firstVisibleRow))
        outfd.write('caretPos             : {0} ({0:#x})\n'.format(listbox.caretPos))
        outfd.write('rowsVisible          : {0} ({0:#x})\n'.format(listbox.rowsVisible))
        outfd.write('itemCount            : {0} ({0:#x})\n'.format(listbox.itemCount))
        outfd.write('stringsStart         : {0:#x} [{1:#x}]\n'.format(
            listbox.stringsStart, task_space.vtop(listbox.stringsStart)))
        outfd.write('stringsLength        : {0} ({0:#x})\n'.format(listbox.stringsLength))
        outfd.write('strings              : {0}\n'.format(listbox.get_text()))

    def dump_combobox(self, dump_to_file, atom_class_name, context, outfd, wnd, task):
        """Process a ComboBox"""
        task_space = task.get_process_address_space()

        addr_wndextra = wnd.v() + self._tagWND_size
        val = wnd.obj_vm.read(addr_wndextra, wnd.cbwndExtra)

        # Build a _COMBOBOX_x?? object from the WndExtra bytes
        if self._profile_is_32bit:
            combobox_type = '_COMBOBOX_x86'
            wndextra = struct.unpack('<I', val)[0]
        elif task.IsWow64:
            combobox_type = '_COMBOBOX_x86'
            wndextra = struct.unpack('<I', val)[0]
        else:
            combobox_type = '_COMBOBOX_x64'
            wndextra = struct.unpack('<Q', val)[0]
        combobox = obj.Object(combobox_type, offset = wndextra, vm = task_space)
        
        outfd.write('{0}\n'.format('*' * 55))
        outfd.write('*** Experimental {0}\n'.format('*' * 38))
        outfd.write('{0}\n'.format('*' * 55))
        outfd.write('Wnd context          : {0}\n'.format(context))
        outfd.write('pointer-to tagWND    : {0:#x} [{1:#x}]\n'.format(wnd.obj_offset, task_space.vtop(wnd.obj_offset)))
        outfd.write('pid                  : {0}\n'.format(wnd.Process.UniqueProcessId))
        outfd.write('process              : {0}\n'.format(wnd.Process.ImageFileName))
        if not self._profile_is_32bit:
            outfd.write('wow64                : {0}\n'.format('Yes' if task.IsWow64 else 'No'))
        outfd.write('atom_class           : {0}\n'.format(atom_class_name))
        outfd.write('address-of cbwndExtra: {0:#x} [{1:#x}]\n'.format(
            wnd.cbwndExtra.obj_offset, task_space.vtop(wnd.cbwndExtra.obj_offset)))
        outfd.write('value-of cbwndExtra  : {0} ({0:#x})\n'.format(wnd.cbwndExtra))
        outfd.write('address-of WndExtra  : {0:#x} [{1:#x}]\n'.format(addr_wndextra, task_space.vtop(addr_wndextra)))
        outfd.write('value-of WndExtra    : {0:#x} [{1:#x}]\n'.format(wndextra, task_space.vtop(wndextra)))
        # if combobox.hWnd != combobox.edithWnd:
        #     outfd.write('handle-of edit       : {0:#x}\n'.format(combobox.edithWnd))
        outfd.write('handle-of combolbox  : {0:#x}\n'.format(combobox.combolboxhWnd))

    def render_text(self, outfd, data):
        """Output the data"""

        # Are we dumping the text to files?
        dump_to_file = self._config.dump_dir != None
        if dump_to_file and not os.path.isdir(self._config.dump_dir):
            debug.error('{0} is not a directory'.format(self._config.dump_dir))

        # Are we removing nulls?
        rm_nulls = not self._config.nulls

        tasks = win32.tasks.pslist(self._addr_space)

        # Build a dict of the tasks, indexed by pid
        the_tasks = {}
        if self._config.pid is None:
            for t in tasks:
                the_tasks[int(t.UniqueProcessId)] = t
        else:
            for t in tasks:
                if self._config.pid == t.UniqueProcessId:
                    the_tasks[int(t.UniqueProcessId)] = t
                    break
        outfd.write('{0} process{1} to check.\n'.format(len(the_tasks), '' if len(the_tasks) == 1 else 'es'))

        # In case the PID's not found
        if len(the_tasks) < 1:
            return

        counts = {}
        for winsta, atom_tables in data:
            for desktop in winsta.desktops():
                for wnd, _level in desktop.windows(desktop.DeskInfo.spwnd):

                    if self._config.pid is None or int(wnd.Process.UniqueProcessId) in the_tasks:

                        atom_class = self.translate_atom(winsta, atom_tables, wnd.ClassAtom)

                        if not isinstance(atom_class, volatility.obj.NoneObject) and \
                            not isinstance(wnd.Process.ImageFileName, volatility.obj.NoneObject):

                            atom_class_name = str(atom_class)
                            context = '{0}\\{1}\\{2}'.format(winsta.dwSessionId, winsta.Name, desktop.Name)

                            if not self._config.experimental_only:

                                # Edit control
                                if atom_class_name.endswith('!Edit'):  # or atom_class_name == 'Edit':
                                    task = the_tasks[int(wnd.Process.UniqueProcessId)]
                                    self.dump_edit(dump_to_file, rm_nulls, atom_class_name, context, outfd, wnd, task)
                                    if 'Edit' in counts:
                                        counts['Edit'] += 1
                                    else:
                                        counts['Edit'] = 1

                            # Experimental options
                            if self._config.experimental or self._config.experimental_only:
                            
                                # Listbox control
                                if atom_class_name.endswith('!Listbox'):  # or atom_class_name.endswith('!ComboLBox')):
                                    task = the_tasks[int(wnd.Process.UniqueProcessId)]
                                    self.dump_listbox(dump_to_file, atom_class_name, context, outfd, wnd, task)
                                    if 'ListBox' in counts:
                                        counts['ListBox'] += 1
                                    else:
                                        counts['ListBox'] = 1

                                # Combobox control
                                elif atom_class_name.endswith('!Combobox'):
                                    task = the_tasks[int(wnd.Process.UniqueProcessId)]
                                    self.dump_combobox(dump_to_file, atom_class_name, context, outfd, wnd, task)
                                    if 'ComboBox' in counts:
                                        counts['ComboBox'] += 1
                                    else:
                                        counts['ComboBox'] = 1
        
        outfd.write('{0}\n'.format('*' * 55))
        for k in counts.keys():
            outfd.write('{0} {1} {2} found.\n'.format(counts[k], k, 'control' if counts[k] == 1 else 'controls'))



# Volatility
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import os, sys, shutil
import subprocess

def run_cmd(args, output_file = None):
    """Run a command through subprocess. 

    @param args: a list of arguments 
    @param output_file: the process's stdout should be redirected here
    """

    print "Running command: {0}".format(" ".join(args))
    if output_file:
        stdout_handle = open(output_file, "w")
    else:
        stdout_handle = subprocess.PIPE
    p = subprocess.Popen(args, stdout = stdout_handle, stderr = subprocess.STDOUT)
    p.wait()
    if output_file:
        stdout_handle.close()
    print "  Retcode: {0}".format(p.returncode)

def generate_profile(temp_dir, volatility_dir, profile_dir, profile):
    """Generate a profile. 

    @param temp_dir: temporary working directory
    @param volatility_dir: path to volatility installation 
    @param profile_dir: where to put finished zip profiles
    @param profile: tuple of information for building the profile
    """

    (full_path, arch, osx_name, version, build) = profile

    # This lets us mount the DMG without a GUI license Y/N prompt
    args = ["/usr/bin/hdiutil", "convert", "-quiet", full_path, "-format", "UDTO", "-o", os.path.join(temp_dir, "test")]
    run_cmd(args)

    args = ["/usr/bin/hdiutil", "attach", "-quiet", "-nobrowse", "-noverify", "-noautoopen", "-mountpoint", "/Volumes/KernelDebugKit", os.path.join(temp_dir, "test.cdr")]
    run_cmd(args)

    dwarf_info = os.path.join(temp_dir, "dwarf.txt")
    # handle the change in filenames in 10.10
    if os.path.isfile("/Volumes/KernelDebugKit/kernel.dSYM"):
        kernel = "/Volumes/KernelDebugKit/kernel.dSYM"
    else:
        kernel = "/Volumes/KernelDebugKit/mach_kernel.dSYM"
    args = ["/usr/bin/dwarfdump", "-arch", arch, "-i", kernel]
    run_cmd(args, output_file = dwarf_info)

    convert_py = os.path.join(volatility_dir, "tools/mac/convert.py")
    new_dwarf = dwarf_info + ".conv"
    args = ["/usr/bin/python", convert_py, dwarf_info, new_dwarf]
    run_cmd(args)
    
    vtypes_file = new_dwarf + ".vtypes"
    args = ["/usr/bin/python", convert_py, new_dwarf]
    run_cmd(args, output_file = vtypes_file)

    symbol_file = dwarf_info + ".symbol.dsymutil"
    # handle the change in filenames in 10.10
    if os.path.isfile("/Volumes/KernelDebugKit/mach_kernel"):
        kernel = "/Volumes/KernelDebugKit/mach_kernel"
    else:
        kernel = "/Volumes/KernelDebugKit/kernel"
    args = ["/usr/bin/dsymutil", "-s", "-arch", arch, kernel]
    run_cmd(args, output_file = symbol_file)    

    profile_name = osx_name + "_" + version
    if arch == "i386":
        profile_name += "_Intel"
    else:
        profile_name += "_AMD"
    profile_name += ".zip"

    zip_file = os.path.join(profile_dir, profile_name)
    args = ["/usr/bin/zip", zip_file, symbol_file, vtypes_file]
    run_cmd(args)

    args = ["/usr/bin/hdiutil", "detach", "/Volumes/KernelDebugKit"]
    run_cmd(args)

    shutil.rmtree(temp_dir)
    os.mkdir(temp_dir)

def main():
    if len(sys.argv) != 5:
        print "Usage: {0} <kit dir> <temp dir> <vol dir> <profile dir>".format(sys.argv[0])
        return 

    profile_runs = []

    for kit in os.listdir(sys.argv[1]):

        try:
            full_path = os.path.join(sys.argv[1], kit)
            file_part = os.path.splitext(kit[len("kernel_debug_kit_"):])[0]
            (version, build) = file_part.split("_")
        except ValueError:
            continue

        if version.startswith("10.5"):
            osx_name = "Leopard"
            profile_runs.append((full_path, "i386", osx_name, version, build))
        elif version.startswith("10.6"):
            osx_name = "SnowLeopard"
            profile_runs.append((full_path, "i386", osx_name, version, build))
            profile_runs.append((full_path, "x86_64", osx_name, version, build))
        elif version.startswith("10.7"):
            osx_name = "Lion"
            profile_runs.append((full_path, "i386", osx_name, version, build))
            profile_runs.append((full_path, "x86_64", osx_name, version, build))
        elif version.startswith("10.8"):
            osx_name = "MountainLion"
            profile_runs.append((full_path, "x86_64", osx_name, version, build))
        elif version.startswith("10.9"):
            osx_name = "Mavericks"
            profile_runs.append((full_path, "x86_64", osx_name, version, build))
        elif version.startswith("10.10"):
            osx_name = "Yosemite"
            profile_runs.append((full_path, "x86_64", osx_name, version, build))

    for profile in profile_runs:
        generate_profile(temp_dir = sys.argv[2], 
                         volatility_dir = sys.argv[3],
                         profile_dir = sys.argv[4],
                         profile = profile)

if __name__ == "__main__":
    main()


# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import volatility.plugins.taskmods as taskmods
from volatility.renderers import TreeGrid

class Cmdline(taskmods.DllList):
    """Display process command-line arguments"""

    def unified_output(self, data):
        # blank header in case there is no shimcache data
        return TreeGrid([("Process", str),
                       ("PID", int),
                       ("CommandLine", str),
                       ], self.generator(data))

    def generator(self, data):
        for task in data:
            cmdline = ""
            if task.Peb:
                cmdline = "{0}".format(str(task.Peb.ProcessParameters.CommandLine or '')).strip()
            yield (0, [str(task.ImageFileName), int(task.UniqueProcessId), str(cmdline)])

    def render_text(self, outfd, data):
        for task in data:
            pid = task.UniqueProcessId

            outfd.write("*" * 72 + "\n")
            outfd.write("{0} pid: {1:6}\n".format(task.ImageFileName, pid))

            if task.Peb:
                outfd.write("Command line : {0}\n".format(str(task.Peb.ProcessParameters.CommandLine or '')))




#
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details. 
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA

"""
@author:       Edwin Smulders
@license:      GNU General Public License 2.0 or later
@contact:      mail@edwinsmulders.eu
"""

import struct
import collections
import itertools

import volatility.plugins.linux.pslist as linux_pslist
import volatility.plugins.linux.proc_maps as linux_proc_maps
import volatility.plugins.linux.common as linux_common
import volatility.plugins.linux.threads as linux_threads


# Because we want to address registers like "registers.eip"
# TODO: replace with linux_info_regs
registers = collections.namedtuple('registers',
                                   [
                                       'r15', 'r14', 'r13', 'r12',
                                       'rbp', 'rbx',
                                       'r11', 'r10', 'r9', 'r8',
                                       'rax', 'rcx', 'rdx',
                                       'rsi', 'rdi',
                                       'unknown',
                                       'rip',
                                       'cs', 'eflags', 'rsp', 'ss'
                                       ])

# TODO: these were the initial registers, they might be valid for x86
# To investigate: view kernel stack using this module
# compare using "info r" in gdb.
# registers = collections.namedtuple('registers',
#                                    ['bla1', 'bla2','bla3','bla4', 'ebx', 'ecx', 'edx',
#                                     'esi', 'edi', 'ebp',
#                                     'eax', 'eds', 'ees',
#                                     'efs', 'egs', 'orig_eax',
#                                     'eip', 'ecs', 'flags',
#                                     'esp', 'ess',
#                                    ]) #test


#registers = collections.namedtuple('registers', ['ebx', 'ecx', 'edx', 'esi', 'edi', 'ebp', 'eax', 'eds', 'ees', 'efs', 'egs', 'orig_eax', 'eip', 'ecs', 'flags', 'esp', 'ess'])

address_size = 8

# Helper functions
def null_list(pages, size):
    """
    Split a section (divided by pages) on 0-bytes.

    @param pages: a list of pages
    @param size: total size of the section
    @return: a list of strings
    """
    res = []
    for page in pages:
        if size > 4096:
            size -= 4096
        else:
            page = page[:size]
            for s in page.split('\0'):
                if s != "":
                    res.append(s)
    return res


def int_list(pages, size):
    """
    Split a range into integers. Will split into words (e.g. 4 or 8 bytes).

    @param pages: a list of pages
    @param size: total size of the section
    @return: a list of word-sized integers
    """
    if address_size == 4:
        fmt = "<L"
    else:
        fmt = "<Q"
    for page in pages:
        curr = 0
        while curr < 4096 and curr < size:
            yield struct.unpack(fmt, page[curr:curr + address_size])[0]
            curr += address_size


# workaround for a bug, it is fixed by now
def _neg_fix(addr):
    return addr
    #return 0xffffffff + addr if addr < 0 else addr


# print as hex (0x12345678)
def print_hex(value):
    """Print a value as in 4 byte hexadecimal."""
    print("0x{:08x}".format(value))


def read_addr_range(start, end, addr_space):
    """
    Read a number of pages.

    @param start: Start address
    @param end: End address
    @param addr_space: The virtual address space
    @return: a list of pages
    """
    pagesize = 4096
    # xrange doesn't support longs :(
    while start < end:
        page = addr_space.zread(start, pagesize)
        yield page
        start += pagesize


def read_null_list( start, end, addr_space):
    """
    Read a number of pages and split it on 0-bytes.

    @param start: Start address
    @param end: End address
    @param addr_space: The virtual address space
    @return: a list of strings
    """
    return null_list(read_addr_range(start, end, addr_space), end - start)

def read_int_list( start, end, addr_space):
    """
    Read a number of pages and split it into integers.

    @param start: Start address
    @param end: End address
    @param addr_space: The virtual address space
    @return: a list of integers.
    """
    return int_list(read_addr_range(start, end, addr_space), end - start)

def read_registers(task, addr_space):
    """
    Read registers from kernel space. Needs to be replaced by the linux_info_regs plugin.

    @param task: The relevant task_struct
    @param addr_space: The kernel address space
    @return: A list of registers (integers)
    """
    return list(
        read_int_list(
            task.thread.sp0 - (21 * address_size),
            task.thread.sp0,
            addr_space
        )
    )

# Main command class
class linux_process_info:
    """ Plugin to gather info for a task/process. Extends pslist. """

    def __init__(self, config, *args, **kwargs):
        linux_common.set_plugin_members(self)
        global address_size
        if self.profile.metadata.get('memory_model', '32bit') == '32bit':
            address_size = 4
        else:
            address_size = 8
        self.get_threads = linux_threads.linux_threads(config).get_threads

    def read_addr_range(self, start, end, addr_space=None):
        """ Read an address range with the task address space as default.

        @param start: Start address
        @param end: End address
        @param addr_space: The address space to read.
        @return: a list of pages
        """
        if addr_space == None:
            addr_space = self.proc_as
        return read_addr_range(start, end, addr_space)

    def calculate(self):

        tasks = linux_pslist.linux_pslist.calculate(self)

        for task in tasks:
            self.task = task

            # Yield a process object
            yield self.analyze(task)

    def read_null_list(self, start, end, addr_space=None):
        """
        Read a number of pages and split it on 0-bytes, with the task address space as default.

        @param start: Start address
        @param end: End address
        @param addr_space: The virtual address space
        @return: a list of strings
        """
        return null_list(self.read_addr_range(start, end, addr_space), end - start)

    def read_int_list(self, start, end, addr_space=None):
        """
        Read a number of pages and split it into integers, with the task addres space as default.

        @param start: Start address
        @param end: End address
        @param addr_space: The virtual address space
        @return: a list of integers.
        """
        return int_list(self.read_addr_range(start, end, addr_space), end - start)

    def analyze(self, task):
        """
        Analyze a task_struct.

        @param task: the task_struct
        @return: a process_info object
        """
        self.proc_as = task.get_process_address_space()
        p = process_info(task)
        p.kernel_as = self.addr_space
        #print(p.stack)
        #linux_volshell.linux_volshell(self._config).render_text.dt('mm_struct')
        #linux_volshell.linux_volshell(self._config).render_text.dt('mm_struct', task.mm)

        # kernel thread?
        p.maps = list(task.get_proc_maps())
        if len(p.maps) == 0:
            return None
        for m in p.maps:
            if m.vm_start <= task.mm.start_stack <= m.vm_end:
                p.vm_stack_low = m.vm_start
                p.vm_stack_high = m.vm_end

        if not p.vm_stack_low:
            last = p.maps[-1]
            p.vm_stack_high = last.vm_end
            p.vm_stack_low = last.vm_start


        p.env = self.read_null_list(_neg_fix(task.mm.env_start), _neg_fix(task.mm.env_end))
        # We care only about the actual stack, not arguments and such
        p.stack = self.read_int_list(_neg_fix(p.vm_stack_low), _neg_fix(task.mm.start_stack))
        p.rest_stack = self.read_int_list(_neg_fix(task.mm.start_stack), _neg_fix(task.mm.env_start))
        p.args = self.read_null_list(_neg_fix(task.mm.arg_start), _neg_fix(task.mm.arg_end))

        reglist = read_registers(task, self.addr_space)
        p.reg = registers(*reglist)
        p.threads = self.get_threads(task)[1]

        return p

    def get_map(self, task, address):
        """
        Get the vm_area to which an address points.

        @param task: the task_struct
        @param address: an address
        @return: a vm_area_struct corresponding to the address
        """
        for m in task.get_proc_maps():
            if m.vm_start <= address <= m.vm_end:
                return m

    def render_text(self, outfd, data):
        self.outfd = outfd
        #pm = linux_proc_maps.linux_proc_maps(self._config)
        #pm.render_text(outfd, pm.calculate())
        #for process in data:
            #mm = process.task.mm
            #print("Heap  Start: {0} End: {1}".format(hex(mm.start_brk), hex(mm.brk)))
            #print("Stack Start: {0} End: {1}".format(hex(mm.start_stack), hex(mm.arg_start)))
            #print("Args  Start: {0} End: {1}".format(hex(mm.arg_start), hex(mm.arg_end)))
            #print("Env   Start: {0} End: {1}".format(hex(mm.env_start), hex(mm.env_end)))
            #self.render_registers(process.reg)
            #main_addrspace = process.task.get_process_address_space()
            #print(main_addrspace, self.addr_space)
            #print(process.task.mm.mm_users.d())
            # outfd.write("{:16s} {:6s} {:18s} {:18s} {:18s}\n".format(
            #     "Thread Name",
            #     "PID",
            #     "task.thread.usersp",
            #     "task.thread.sp0",
            #     "Register esp"
            # ))
    #         for task in process.threads:
    #             #print(task.thread.d())
    #             #thread_p = self.analyze(task)
    #             #proc_as = task.get_process_address_space()
    #             map = self.get_map(task, task.thread.usersp)
    #             map_flags = map.vm_flags
    #             if not map_flags.is_readable() or not map_flags.is_writable() or map.vm_file != 0:
    #                 outfd.write("File {}\n".format(map.vm_file))
    #                 outfd.write("{:16s} {:6s} 0x{:016x} 0x{:016x} 0x{:016x} 0x{:016x} 0x{:016x}\n".format(
    #                     task.comm,
    #                     str(task.pid),
    #                     task.thread.usersp,
    #                     task.thread.sp0,
    #                     #thread_p.reg.esp,
    #                     0,
    #                     map.vm_start,
    #                     map.vm_end
    # #                    task.get_process_address_space().vtop(reg.esp),
    # #                    main_addrspace.vtop(reg.esp)
    #
    #                 ))
    #             else:
    #                 outfd.write("Map looked like stack\n")
            #cProfile.run("self.render_list(process.get_pointers())")
            #self.render_list(process.get_unique_data_pointers())
            #self.render_list(process.get_data_pointers_from_heap())
            #self.render_annotated_list(process.annotated_stack())
            # still broken
            #self.render_stack_frames(process.stack_frames)
            #exit(0)
            #self.render_annotated_list(stack)kleutvieul

    def render_stack_frames(self, stack_frames):
        """
        Render stackframes (old code)
        @param stack_frames: a list of stackframes
        @return: None
        """
        for stack_frame in stack_frames:
            self.table_header(self.outfd, [('Stack Frame', '16'), ('Value', '[addrpad]')])
            self.table_row(self.outfd, "Frame Number", stack_frame.frame_number)
            self.table_row(self.outfd, "Offset", stack_frame.offset)
            self.table_row(self.outfd, "Return Address", stack_frame.ret)

    def render_registers(self, reg):
        """
        Render a registers named tuple.
        @param reg: registers named tuple
        @return: None
        """
        self.table_header(self.outfd, [('Register', '8'), ('Value', '[addrpad]')])
        for k in reg._fields:
            self.table_row(self.outfd, k, getattr(reg, k))

    def render_list(self, l):
        """
        Render an address list
        @param l: address list
        @return: None
        """
        self.table_header(self.outfd, [('Address', '[addrpad]'), ('Value', '[addrpad]')])
        for address, value in l:
            self.table_row(self.outfd, address, value)

    def render_annotated_list(self, ann_list):
        """
        Render a list including annotations.
        @param ann_list: a 3-tuple list
        @return: None
        """
        self.table_header(self.outfd, [('Address', '[addrpad]'), ('Value', '[addrpad]'), ('Annotation', '50')])
        for (address, value, annotation) in ann_list:
            self.table_row(self.outfd, address, value, annotation)


class process_info(object):
    """
    A class to collect various information about a process/task. Includes helper functions to detect pointers.
    """

    def __init__(self, task):
        self.task = task
        self.mm = task.mm


        ####
        # obj.CType is really slow (__getattr__), so we do this
        self.mm_brk = _neg_fix(self.mm.brk)
        self.mm_end_code = _neg_fix(self.mm.end_code)
        self.mm_end_data = _neg_fix(self.mm.end_data)
        self.mm_env_end = _neg_fix(self.mm.env_end)
        self.mm_start_brk = _neg_fix(self.mm.start_brk)
        self.mm_start_code = _neg_fix(self.mm.start_code)
        self.mm_start_data = _neg_fix(self.mm.start_data)
        ####

        self.proc_as = task.get_process_address_space()
        self.kernel_as = None
        self.env = None
        self.rest_stack = None
        self.args = None
        self.vm_stack_low = None
        self.vm_stack_high = None
        self.stack_frames = None
        #self.threads = None
        self.thread_stacks = None
        self.thread_stack_ranges = None

        # properties
        self._stack = None
        self._threads = None
        self._reg = None
        self._real_stack_low = None
        self._maps = None
        self._exec_maps = None
        self._exec_maps_ranges = None

        self.is_pointer_dict = dict(stack=self.is_stack_pointer, heap=self.is_heap_pointer,
                                    constant=self.is_constant_pointer, code=self.is_code_pointer)

    @property
    def maps(self):
        """
        @return: the vm_area maps list.
        """
        return self._maps

    @maps.setter
    def maps(self, value):
        """
        Setter for maps. Also initializes some other values.
        @param value: The list of vm_area maps
        @return: None
        """
        self._maps = value
        self._exec_maps = []
        self._exec_maps_ranges = []
        for m in self._maps:
            if m.vm_flags.is_executable():
                self._exec_maps.append(m)
                self._exec_maps_ranges.append((m.vm_start, m.vm_end))

    @property
    def reg(self):
        """
        @return: the registers named tuple for this process
        """
        return self._reg

    @reg.setter
    def reg(self, value):
        """
        Setter for reg.
        @param value: The named tuple for registers.
        @return: None
        """
        self._reg = value
        #self._generate_stack_frames()

    @property
    def stack(self):
        """
        Get the _list_ of stack values (old code).
        @return: stack integer list.
        """
        return self._stack

    @stack.setter
    def stack(self, value):
        """
        Set the stack list (old code).
        @param value: a list of integers.
        @return: None
        """
        self._stack = list(value)
        self._calculate_stack_offset()
        #self._generate_thread_stack_list()
        #self._generate_stack_frames()

    @property
    def threads(self):
        """
        Get the list of threads for this process.
        @return: a list of task_structs (threads).
        """
        return self._threads

    @threads.setter
    def threads(self, value):
        """
        Set the list of threads. Initializes the list of register tuples for these threads.
        @param value: The list of task_structs.
        @return: None
        """
        self._threads = value
        self.thread_registers = self._find_thread_registers()
        self._generate_thread_stack_list()

    def _find_thread_registers(self):
        """
        Reads the registers from the kernel stack for all threads.
        @return: list of tuple of registers.
        """
        reglist = []
        for task in self.threads:
            reglist.append(registers(*read_registers(task, self.kernel_as)))
        return reglist


    def get_stack_value(self, address):
        """
        Read a value from the stack, by using the stack list (old code).
        @param address: The address to read.
        @return: The word at this address.
        """
        return self.stack[self.get_stack_index(address)]

    def get_stack_index(self, address):
        """
        Calculates the index on the stack list given an address.
        @param address: The address to find
        @return: an index into process_info.stack
        """
        return (address - self.vm_stack_low) / address_size

    def _generate_thread_stack_list(self):
        """
        Makes a list of the stack vm areas for all threads. Uses the register contents.
        @return: None
        """
        if not self.threads or not self.maps:
            self.thread_stacks = None
        else:
            thread_sps = [self.thread_registers[i].rsp for i, task in enumerate(self.threads)]
            thread_sps.sort()
            self.thread_stacks = []
            self.thread_stack_ranges = []
            #for i in range(len(thread_sps)):
            i = 0
            for m in self.maps:
                if i < len(thread_sps) and m.vm_start <= thread_sps[i] <= m.vm_end:
                    self.thread_stacks.append(m)
                    self.thread_stack_ranges.append((m.vm_start, m.vm_end))
                    i+=1

    def _calculate_stack_offset(self):
        """
        Calculates the absolute bottom of the stack (everything below is 0). (old code)
        @return: The lowest stack address.
        """
        offset = self.vm_stack_low
        for i in self._stack:
            if i != 0:
                self._real_stack_low = offset
                break
            offset += 4
        return self._real_stack_low

    def annotate_addr_list(self, l, offset=None, skip_zero=True):
        """
        Annotates a list of addresses with some basic pointer and register information (old code).
        @param l: list of addresses.
        @param offset: Offset of the list
        @param skip_zero:
        @return: An annotated list
        """
        if offset == None:
            offset = self.vm_stack_low
        for value in l:
            if value != 0:
                skip_zero = False
            pointer_type = self.get_pointer_type(value)
            annotation = ""
            if pointer_type != None:
                annotation = pointer_type + " pointer"
            if offset == self.reg.esp:
                annotation += " && register esp"
            elif offset == self.reg.ebp:
                annotation += " && register ebp"
            if not skip_zero:
                yield (offset, value, annotation)
            offset += 4

    def is_stack_pointer(self, addr):
        """
        Check if addr is a pointer to the (main) stack.
        @param addr: An address
        @return: True or False
        """
        return self.vm_stack_low <= addr <= self.mm_env_end

    def is_thread_stack_pointer(self, addr):
        """
        Check if addr is a pointer to a thread stack.
        FIXME: enable checking a specific stack.
        @param addr: An address
        @return: True or False
        """
        for m_start, m_end in self.thread_stack_ranges:
            if m_start <= addr <= m_end:
                return True
        return False

    def is_heap_pointer(self, addr):
        """
        Check if addr is a pointer to the heap.
        @param addr: An address
        @return: True or False
        """
        return self.mm_start_brk <= addr <= self.mm_brk

    def is_constant_pointer(self, addr):
        """
        Check if addr is a pointer to a program constant
        @param addr: An address
        @return: True of False
        """
        return self.mm_start_data <= addr <= self.mm_end_data

    def is_program_code_pointer(self, addr):
        """
        Check if addr is a pointer to the program code
        @param addr: An address
        @return: True of False
        """
        return self.mm_start_code <= addr <= self.mm_end_code

    def is_library_code_pointer(self, addr):
        """
        Check if addr is a pointer to library code
        @param addr: An address
        @return: True or False
        """
        return self.is_code_pointer(addr) and not self.is_program_code_pointer(addr)

    def is_code_pointer(self, addr):
        """
        Check if addr is a pointer to an executable section of memory
        @param addr: An address
        @return: True or False
        """
        for m_start, m_end in self._exec_maps_ranges:
            if m_start <= addr <= m_end:
                return True
        return False

    def is_data_pointer(self, addr):
        """
        Check if addr points to data (not code)
        @param addr: An address
        @return: True or False
        """
        return self.is_heap_pointer(addr) or self.is_stack_pointer(addr) or self.is_constant_pointer(addr) or self.is_thread_stack_pointer(addr)

    def is_pointer(self, addr, space=None):
        """
        Check if addr is any sort of pointer
        @param addr: An address
        @param space: A choice of stack, heap, etc
        @return: True or False
        """
        if not space:
            for func in self.is_pointer_dict.itervalues():
                if func(addr):
                    return True
            return False
        else:
            return self.is_pointer_dict[space]

    def get_map_by_name(self, name, permissions='r-x'):
        """
        Find a memory mapping (vm_area) by its name (not exact match). Optionally, check permissions.
        @param name: The mapped name to find.
        @param permissions: Permissions in 'rwx' format
        @return: A (vm_start, vm_end, libname) tuple or None
        """
        # We use this to find libc
        for vma in self.task.get_proc_maps():
            libname = linux_common.get_path(self.task, vma.vm_file)
            # just look for partial name
            if str(vma.vm_flags) == permissions and name in libname:
                return vma.vm_start, vma.vm_end, libname
        return None

    def get_unique_data_pointers(self):
        """
        A filter over get_data_pointers() to get only unique values.
        @return: A iterator of pointers.
        """
        return self.get_unique_pointers(self.get_data_pointers())

    def get_unique_pointers(self, pointer_iter=None):
        """
        Filter an iterator to only return unique values.
        @param pointer_iter: The pointer iterator to use. If None, use get_pointers().
        @return: An iterator of unique pointers
        """
        if pointer_iter == None:
            pointer_iter = self.get_pointers()

        store = []
        
        for address, value in pointer_iter:
            if value not in store:
                yield address, value
                store.append(value)

    def get_data_pointers(self):
        """
        Calls get_pointers with self.is_data_pointer as a filter.
        @return: An iterator of pointers
        """
        return self.get_pointers(self.is_data_pointer)

    def get_pointers(self, cond=None, space=None):
        """
        Finds pointers given a condition and a space. (old code)
        @param cond: The type of pointer to filter, defaults to self.is_pointer
        @param space: The list of values to use, defaults to self.stack
        @return: An iterator of addresses and their values.
        """
        if cond == None:
            cond = self.is_pointer
        if space == None:
            space = self.stack
        address = self.vm_stack_low
        for value in space:
            if value != 0 and cond(value):
                yield address, value
            address += address_size

    def get_data_pointers_from_heap(self):
        """
        Find data pointers on the heap, very slow.
        @return: An iterator of pointers
        """
        return self.get_pointers(
            cond = self.is_data_pointer,
            space = read_int_list(self.mm_start_brk, self.mm_brk, self.proc_as)
        )

    def get_data_pointers_from_map(self, m):
        """
        Find data pointers from a specific mapping, very slow.
        @param m: The vm_area map
        @return: An iterator of pointers
        """
        return self.get_pointers(
            cond = self.is_data_pointer,

            space = read_int_list(m.vm_start, m.vm_end, self.proc_as)
        )

    def get_data_pointers_from_threads(self):
        """
        Find data pointers from all threads
        @return: An iterator of all pointers on thread stacks
        """
        iterators = [self.get_data_pointers_from_map(m) for m in self.thread_stacks]
        return self.get_unique_pointers(itertools.chain(*iterators))

    def get_pointers_from_stack(self):
        """
        Find pointers on the main stack
        @return: An iterator of pointers
        """
        return self.get_pointers(space=self.stack)

    def get_pointer_type(self, addr):
        """
        Determine the pointer type for a specific address.
        @param addr: An address.
        @return: String pointer type
        """
        for k, v in self.is_pointer_dict.iteritems():
            if v(addr):
                return k
        return None

    def annotated_stack(self):
        """
        Uses annotate_addr_list() to annotate the stack.
        @return: An annotated address list of the stack
        """
        return self.annotate_addr_list(self._stack)






# Volatility
#
# Authors
# Michael Cohen <scudette@users.sourceforge.net>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""pstree example file"""
from volatility import renderers
from volatility.renderers.basic import Address

import volatility.win32.tasks as tasks
import volatility.utils as utils
import volatility.plugins.common as common
import volatility.cache as cache
import volatility.obj as obj
import volatility.debug as debug

#pylint: disable-msg=C0111

class ProcessAuditVTypes(obj.ProfileModification):
    before = ["WindowsVTypes"]
    conditions = {'os': lambda x: x == 'windows'}
    def modification(self, profile):
        profile.vtypes.update({
            '_SE_AUDIT_PROCESS_CREATION_INFO' : [ 0x4, {
            'ImageFileName' : [ 0x0, ['pointer', ['_OBJECT_NAME_INFORMATION']]],
            }],
            '_OBJECT_NAME_INFORMATION' : [ 0x8, {
            'Name' : [ 0x0, ['_UNICODE_STRING']],
            }]})

class PSTree(common.AbstractWindowsCommand):
    """Print process list as a tree"""

    text_sort_column = "Pid"

    def find_root(self, pid_dict, pid):
        # Prevent circular loops.
        seen = set()

        while pid in pid_dict and pid not in seen:
            seen.add(pid)
            pid = int(pid_dict[pid].InheritedFromUniqueProcessId)

        return pid

    def generator(self, data):
        def draw_branch(level, inherited_from):
            for task in data.values():
                if task.InheritedFromUniqueProcessId == inherited_from:

                    row = [Address(task.obj_offset),
                           str(task.ImageFileName or ''),
                           int(task.UniqueProcessId),
                           int(task.InheritedFromUniqueProcessId),
                           int(task.ActiveThreads),
                           int(task.ObjectTable.HandleCount),
                           str(task.CreateTime)]

                    if self._config.VERBOSE:
                        row += [str(task.SeAuditProcessCreationInfo.ImageFileName.Name or '')]
                        process_params = task.Peb.ProcessParameters
                        if not process_params:
                            row += [str("-"), str("-")]
                        else:
                            row += [str(process_params.CommandLine or ''),
                                    str(process_params.ImagePathName or '')]
                    yield (level, row)

                    try:
                        del data[int(task.UniqueProcessId)]
                    except KeyError:
                        debug.warning("PID {0} PPID {1} has already been seen".format(task.UniqueProcessId,
                                                                                      task.InheritedFromUniqueProcessId))

                    for item in draw_branch(level + 1, task.UniqueProcessId):
                        yield item

        while len(data.keys()) > 0:
            keys = data.keys()
            root = self.find_root(data, keys[0])
            for item in draw_branch(0, root):
                yield item

    def unified_output(self, data):

        cols = [("Offset", Address),
                ("Name", str),
                ("Pid", int),
                ("PPid", int),
                ("Thds", int),
                ("Hnds", int),
                ("Time", str)]

        if self._config.VERBOSE:
            cols += [("Audit", str),
                     ("Cmd", str),
                     ("Path", str)]

        tg = renderers.TreeGrid(cols, self.generator(data))
        return tg

    def render_text(self, outfd, data):
        self.table_header(outfd, 
                         [("Name", "<50"), 
                          ("Pid", ">6"),
                          ("PPid", ">6"),
                          ("Thds", ">6"),
                          ("Hnds", ">6"),
                          ("Time", "")])

        def draw_branch(pad, inherited_from):
            for task in data.values():
                if task.InheritedFromUniqueProcessId == inherited_from:

                    first_column = "{0} {1:#x}:{2:20}".format(
                                        "." * pad, 
                                        task.obj_offset, 
                                        str(task.ImageFileName or '')
                                        )

                    self.table_row(outfd, 
                        first_column,
                        task.UniqueProcessId,
                        task.InheritedFromUniqueProcessId,
                        task.ActiveThreads,
                        task.ObjectTable.HandleCount,
                        task.CreateTime)

                    if self._config.VERBOSE:
                        outfd.write("{0}    audit: {1}\n".format(
                                ' ' * pad, str(task.SeAuditProcessCreationInfo.ImageFileName.Name or '')))
                        process_params = task.Peb.ProcessParameters
                        if process_params:
                            outfd.write("{0}    cmd: {1}\n".format(
                                ' ' * pad, str(process_params.CommandLine or '')))
                            outfd.write("{0}    path: {1}\n".format(
                                ' ' * pad, str(process_params.ImagePathName or '')))

                    try:
                        del data[int(task.UniqueProcessId)]
                    except KeyError:
                        debug.warning("PID {0} PPID {1} has already been seen".format(task.UniqueProcessId, task.InheritedFromUniqueProcessId))

                    draw_branch(pad + 1, task.UniqueProcessId) 

        while len(data.keys()) > 0:
            keys = data.keys()
            root = self.find_root(data, keys[0])
            draw_branch(0, root)

    @cache.CacheDecorator(lambda self: "tests/pstree/verbose={0}".format(self._config.VERBOSE))
    def calculate(self):

        ## Load a new address space
        addr_space = utils.load_as(self._config)

        return dict(
                (int(task.UniqueProcessId), task)
                for task in tasks.pslist(addr_space)
                )



# Volatility
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details. 
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization:
"""

import volatility.obj as obj
import volatility.debug as debug
import volatility.plugins.linux.common as linux_common
import volatility.plugins.linux.pslist as linux_pslist
from volatility.renderers import TreeGrid
from volatility.renderers.basic import Address

class linux_library_list(linux_pslist.linux_pslist):
    """ Lists libraries loaded into a process """

    def calculate(self):
        linux_common.set_plugin_members(self)
   
        tasks = linux_pslist.linux_pslist.calculate(self)

        for task in tasks:
            for mapping in task.get_libdl_maps():
                if mapping.l_name == "" or mapping.l_addr == 0:
                    continue

                yield task, mapping

    def unified_output(self, data):
        return TreeGrid([("Task", str),
                       ("Pid", int),
                       ("LoadAddress", Address),
                       ("Path", str)],
                        self.generator(data))

    def generator(self, data):
        for task, mapping in data:
            yield (0, [str(task.comm), int(task.pid), Address(mapping.l_addr), str(mapping.l_name)])

    def render_text(self, outfd, data):
        self.table_header(outfd, [("Task", "16"),
                          ("Pid", "8"),
                          ("Load Address", "[addrpad]"),
                          ("Path", ""),
                          ])
  
        for task, mapping in data:
            self.table_row(outfd, task.comm, task.pid, mapping.l_addr, mapping.l_name)



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import volatility.obj as obj
import volatility.plugins.mac.common as common
import volatility.plugins.mac.list_zones as list_zones
import volatility.plugins.mac.pslist as pslist

class mac_dead_vnodes(pslist.mac_pslist):
    """ Lists freed vnode structures """

    def calculate(self):
        common.set_plugin_members(self)
    
        zones = list_zones.mac_list_zones(self._config).calculate()

        for zone in zones:
            name = str(zone.zone_name.dereference())
            if name == "vnodes":
                vnodes = zone.get_free_elements("vnode")        
                for vnode in vnodes:
                    yield vnode

    def render_text(self, outfd, data):
        for vnode in data:
            path = vnode.full_path()
            if path:
                outfd.write("{0:s}\n".format(path))






# Volatility
# Copyright (C) 2008-2013 Volatility Foundation
# Copyright (c) 2008 Brendan Dolan-Gavitt <bdolangavitt@wesleyan.edu>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       AAron Walters and Brendan Dolan-Gavitt
@license:      GNU General Public License 2.0
@contact:      awalters@4tphi.net,bdolangavitt@wesleyan.edu
@organization: Volatility Foundation
"""

#pylint: disable-msg=C0111

import volatility.utils as utils 
import volatility.poolscan as poolscan
import volatility.plugins.common as common
import volatility.plugins.bigpagepools as bigpools
from volatility.renderers import TreeGrid
from volatility.renderers.basic import Address

class PoolScanHive(poolscan.PoolScanner):
    """Pool scanner for registry hives"""

    def __init__(self, address_space):
        poolscan.PoolScanner.__init__(self, address_space)
        self.struct_name = "_CMHIVE"
        self.pooltag = "CM10"
        size = self.address_space.profile.get_obj_size("_CMHIVE")
        self.checks = [ 
                   ('CheckPoolSize', dict(condition = lambda x: x >= size)),
                   ]

class HiveScan(common.AbstractScanCommand):
    """Pool scanner for registry hives"""

    scanners = [PoolScanHive]
    # Declare meta information associated with this plugin

    meta_info = dict(
        author = 'Brendan Dolan-Gavitt',
        copyright = 'Copyright (c) 2007,2008 Brendan Dolan-Gavitt',
        contact = 'bdolangavitt@wesleyan.edu',
        license = 'GNU General Public License 2.0',
        url = 'http://moyix.blogspot.com/',
        os = 'WIN_32_XP_SP2',
        version = '1.0',
        )

    def calculate(self):
        addr_space = utils.load_as(self._config)

        metadata = addr_space.profile.metadata
        version = (metadata.get("major", 0), metadata.get("minor", 0))
        arch = metadata.get("memory_model", "32bit")

        if version >= (6, 3) and arch == "64bit":
            for pool in bigpools.BigPagePoolScanner(addr_space).scan(["CM10"]):
                yield pool.Va.dereference_as("_CMHIVE")
        else:
            for result in self.scan_results(addr_space):
                yield result

    def unified_output(self, data):
        return TreeGrid([("Offset(P)", Address)],
                        self.generator(data))

    def generator(self, data):
        for hive in data:
            yield(0, [Address(hive.obj_offset)])

    def render_text(self, outfd, data):
        self.table_header(outfd, [('Offset(P)', '[addrpad]')])
        for hive in data:
            self.table_row(outfd, hive.obj_offset)



__author__ = 'mike'

import volatility.utils as utils

class Bytes(bytes):
    """String class to allow us to encode binary data"""
    def __new__(cls, data):
        if data == None:
            return str.__new__(cls, "-")
        return str.__new__(cls, data.encode("hex"))

class Address(long):
    """Integer class to allow renderers to differentiate between addresses and numbers"""
    def __new__(cls, number):
        return long.__new__(cls, number)


class Address64(long):
    """Integer class to allow renderers to differentiate between addresses and numbers"""

    def __new__(cls, number):
        return long.__new__(cls, number)


class Hex(long):
    """Integer class to allow renderers to differentiate between addresses and numbers"""

    def __new__(cls, number):
        return long.__new__(cls, number)


class Renderer(object):
    def render(self, outfd, grid):
        """Renders the content, ideally to outfd, but this is not strictly necessary"""



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""
import re

import volatility.obj as obj
import volatility.plugins.linux.common as linux_common
import volatility.plugins.linux.lsmod  as linux_lsmod
from volatility.renderers import TreeGrid
from volatility.renderers.basic import Address

class linux_hidden_modules(linux_common.AbstractLinuxCommand):
    """Carves memory to find hidden kernel modules"""

    def walk_modules_address_space(self, addr_space):
        list_mods = [x[0].obj_offset for x in linux_lsmod.linux_lsmod(self._config).calculate()]

        # this for is for pre-2008 kernels:
        # https://git.kernel.org/cgit/linux/kernel/git/torvalds/linux.git/commit/kernel/module.c?id=3a642e99babe0617febb6f402e1e063479f489db)
        if addr_space.profile.get_symbol("module_addr_min") == None:
            return

        min_addr_sym = obj.Object("unsigned long", offset = addr_space.profile.get_symbol("module_addr_min"), vm = addr_space)
        max_addr_sym = obj.Object("unsigned long", offset = addr_space.profile.get_symbol("module_addr_max"), vm = addr_space)

        min_addr = min_addr_sym & ~0xfff
        max_addr = (max_addr_sym & ~0xfff) + 0x1000
        
        scan_buf = ""
        llen = max_addr - min_addr
        
        allfs = "\xff" * 4096 
        
        memory_model = self.addr_space.profile.metadata.get('memory_model', '32bit')
        if memory_model == '32bit':
            minus_size = 4
        else:
            minus_size = 8
 
        check_bufs = []
        replace_bufs = []
        
        check_nums = [3000, 2800, 2700, 2500, 2300, 2100, 2000, 1500, 1300, 1200, 1024, 512, 256, 128, 96, 64, 48, 32, 24, 16, 12, 9]
        if minus_size == 4:
            check_nums = check_nums + [8, 6, 5]

        for num in check_nums:
            check_bufs.append("\x00" * num)        
            replace_bufs.append(("\xff" * (num-minus_size)) + "\x00" * minus_size)

        for page in range(min_addr, max_addr, 4096):
            to_append = allfs

            tmp = addr_space.read(page, 4096)
            if tmp:
                non_zero = False
                for t in tmp:
                    if t != "\x00":
                        non_zero = True
                        break

                if non_zero:
                    for i in range(len(check_nums)):
                        tmp = tmp.replace(check_bufs[i], replace_bufs[i])
                    to_append = tmp

            scan_buf = scan_buf + to_append

        for cur_addr in re.finditer("(?=(\x00\x00\x00\x00|\x01\x00\x00\x00|\x02\x00\x00\x00))", scan_buf):
            mod_addr = min_addr + cur_addr.start()

            if mod_addr in list_mods:
                continue

            m = obj.Object("module", offset = mod_addr, vm = addr_space)

            if m.is_valid():
                yield m

    def calculate(self):
        linux_common.set_plugin_members(self)

        for mod in self.walk_modules_address_space(self.addr_space):
            yield mod

    def unified_output(self, data):
        return TreeGrid([("Offset(V)", Address),
                       ("Name", str)],
                        self.generator(data))

    def generator(self, data):
        for module in data:
            yield (0, [Address(module.obj_offset), str(module.name)])

    def render_text(self, outfd, data):
        self.table_header(outfd, [("Offset (V)", "[addrpad]"), ("Name", "")])

        for module in data:
            self.table_row(outfd, module.obj_offset, str(module.name))




# Volatility
# Copyright (C) 2007-2014 Volatility Foundation
# Copyright (C) 2014 Michael Hale Ligh <michael.ligh@mnin.org>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import volatility.obj as obj 
import volatility.plugins.gui.constants as consts
import volatility.plugins.gui.win32k_core as win32k_core
import volatility.plugins.gui.vtypes.win7_sp0_x86_vtypes_gui as win7_sp0_x86_vtypes_gui
import volatility.plugins.gui.vtypes.win7_sp0_x64_vtypes_gui as win7_sp0_x64_vtypes_gui

class _RTL_ATOM_TABLE_ENTRY(win32k_core._RTL_ATOM_TABLE_ENTRY):
    """A class for atom table entries"""

    @property
    def Flags(self):
        return self.Reference.Flags

    @property
    def ReferenceCount(self):
        return self.Reference.ReferenceCount

class Win8x86Gui(obj.ProfileModification):

    before = ["XP2003x86BaseVTypes", "Win32Kx86VTypes", "AtomTablex86Overlay", "Win32KCoreClasses"]

    conditions = {'os': lambda x: x == 'windows',
                  'memory_model': lambda x: x == '32bit',
                  'major': lambda x: x == 6,
                  'minor': lambda x: x > 1}

    def modification(self, profile):

        profile.vtypes.update(win7_sp0_x86_vtypes_gui.win32k_types)
        profile.object_classes.update({'_RTL_ATOM_TABLE_ENTRY': _RTL_ATOM_TABLE_ENTRY})

        profile.merge_overlay({
            'tagWINDOWSTATION' : [ None, {
            ## ForceEmptyClipboard
            ## lea     eax, [esi+28h]
            ## call    @HMAssignmentUnlock@4 ; HMAssignmentUnlock(x)
            ## lea     eax, [esi+24h]
            ## call    @HMAssignmentUnlock@4 ; HMAssignmentUnlock(x)
            'spwndClipOwner': [0x28, ['pointer', ['tagWND']]],
            'spwndClipViewer': [0x24, ['pointer', ['tagWND']]],

            ## _EnumClipboardFormats
            ## mov     ecx, [esi+30h]
            'pClipBase' : [ 0x30, ['pointer', ['array', lambda x : x.cNumClipFormats, ['tagCLIP']]]], 

            ## xxxEmptyClipboard
            ## mov     eax, [ebx+34h]
            'cNumClipFormats': [0x34, ['unsigned long']],

            ## xxxEmptyClipboard
            ## call    @HMAssignmentLock@8 ; HMAssignmentLock(x,x)
            ## inc     dword ptr [ebx+38h]
            'iClipSerialNumber': [0x38, ['unsigned long']],

            ## xxxCreateWindowStation
            ## lea     edi, [edi+48h]
            ## call    _CreateGlobalAtomTab
            'pGlobalAtomTable': [ 0x48, ['pointer', ['void']]],
            }],

            '_HANDLEENTRY': [ None, {
            'bType': [ None, ['Enumeration', dict(target = 'unsigned char', choices = consts.HANDLE_TYPE_ENUM_SEVEN)]],
            }],

            'tagCLIP': [ 20, {
            'fmt' : [ None, ['Enumeration', dict(target = 'unsigned long', choices = consts.CLIPBOARD_FORMAT_ENUM)]],
            }],

            'tagTHREADINFO': [ None, {
            ## xxxCreateWindowStation
            ## mov     ebx, _gptiCurrent
            ## mov     eax, [ebx+0C4h]
            'ppi': [0xc4, ['pointer', ['tagPROCESSINFO']]],

            ## zzzReattachThreads
            ## lea     ebx, [edi-158h]
            ## mov     ecx, [ebx+130h]
            'PtiLink': [0x158, ['_LIST_ENTRY']],
            }],

            'tagDESKTOP': [ None, {
            ## ParseDesktop
            ## mov     edi, [edi+8]
            ## test    edi, edi
            'rpdeskNext': [8, ['pointer', ['tagDESKTOP']]],

            ## DestroyDesktop
            ## mov     ebx, [ebp+arg_0]
            ## mov     eax, [ebx+0Ch]
            'rpwinstaParent': [0xc, ['pointer', ['tagWINDOWSTATION']]],

            ## DesktopAlloc
            ## mov     eax, [eax+3Ch]
            ## push    edi
            ## push    [ebp+Size]      ; Size
            'pheapDesktop': [0x3c, ['pointer', ['tagWIN32HEAP']]],

            ### xxxCreateDesktopEx2
            ## add     eax, 58h
            ## mov     [eax+4], eax
            ## mov     [eax], eax    
            'PtiList': [0x58, ['_LIST_ENTRY']],
            }],

            '_RTL_ATOM_TABLE': [ None, {
            'NumBuckets': [ 0x14, ['unsigned long']],
            'Buckets': [ 0x18, ['array', lambda x : x.NumBuckets,
                ['pointer', ['_RTL_ATOM_TABLE_ENTRY']]]],
            }],

        })

class Win8x64Gui(obj.ProfileModification):

    before = ["Win32KCoreClasses"]

    conditions = {'os': lambda x: x == 'windows',
                  'memory_model': lambda x: x == '64bit',
                  'major': lambda x: x == 6,
                  'minor': lambda x: x > 1}

    def modification(self, profile):

        profile.vtypes.update(win7_sp0_x64_vtypes_gui.win32k_types)
        profile.object_classes.update({'_RTL_ATOM_TABLE_ENTRY': _RTL_ATOM_TABLE_ENTRY})

        profile.merge_overlay({

            'tagWINDOWSTATION': [ None, { 
            ## _EnumClipboardFormats
            ## mov     rcx, [rdi+60h]
            ## test    rcx, rcx
            'pClipBase' : [ 0x60, ['pointer', ['array', lambda x : x.cNumClipFormats, ['tagCLIP']]]], 

            ## xxxEmptyClipboard
            ## mov     ebp, [rbx+68h]
            'cNumClipFormats': [0x68, ['unsigned long']],

            ## xxxEmptyClipboard
            ## call    HMAssignmentLock
            ## inc     dword ptr [rbx+6Ch]
            'iClipSerialNumber': [0x6c, ['unsigned long']],

            ## xxxCreateWindowStation
            ## add     rcx, 88h
            ## call    CreateGlobalAtomTable 
            'pGlobalAtomTable': [ 0x88, ['pointer', ['void']]],
            }],

            'tagDESKTOP': [ None, {
            ## ParseDesktop
            ## mov     rdi, [rdi+10h]
            'rpdeskNext': [0x10, ['pointer', ['tagDESKTOP']]],

            ## DestroyDesktop
            ## mov     eax, [rcx+20h]
            ## mov     rdi, [rcx+18h]
            'rpwinstaParent': [0x18, ['pointer', ['tagWINDOWSTATION']]],

            ## DesktopAlloc
            ## mov     rcx, [rcx+78h]
            ## mov     r8d, edx
            ## xor     edx, edx
            'pheapDesktop': [0x78, ['pointer', ['tagWIN32HEAP']]],

            ### xxxCreateDesktopEx2
            ## add     rax, 0A0h
            ## mov     [rax+8], rax 
            'PtiList': [0xA0, ['_LIST_ENTRY']],
            }],

            'tagTHREADINFO': [ None, {
            ## xxxCreateWindowStation
            ## mov     rsi, cs:gptiCurrent
            ## mov     rax, [r14+10h]
            ## mov     rcx, [rax+170h]
            'ppi': [0x170, ['pointer', ['tagPROCESSINFO']]],

            ## zzzReattachThreads
            ## lea     rsi, [rdi-280h]
            ## mov     rdx, [rsi+230h] ; struct tagQ *
            ## cmp     rdx, [rsi+178h]
            'PtiLink': [0x280, ['_LIST_ENTRY']],
            }],

            'tagCLIP': [ None, {
            'fmt' : [ None, ['Enumeration', dict(target = 'unsigned long', choices = consts.CLIPBOARD_FORMAT_ENUM)]],
            }],

            '_RTL_ATOM_TABLE': [ None, {
            'NumBuckets': [ 0x1C, ['unsigned long']],
            'Buckets': [ 0x20, ['array', lambda x : x.NumBuckets,
                ['pointer', ['_RTL_ATOM_TABLE_ENTRY']]]],
            }],

            '_HANDLEENTRY': [ None, {
            'bType': [ None, ['Enumeration', dict(target = 'unsigned char', choices = consts.HANDLE_TYPE_ENUM_SEVEN)]],
            }],

            })


# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Jamie Levy (gleeda)
@license:      GNU General Public License 2.0
@contact:      jamie@memoryanalysis.net
@organization: Volatility Foundation
"""

import volatility.plugins.taskmods as taskmods
from volatility.renderers import TreeGrid
from volatility.renderers.basic import Address

class JobLinks(taskmods.DllList):
    """ Print process job link information"""
    def __init__(self, config, *args, **kwargs):
        taskmods.DllList.__init__(self, config, *args, **kwargs)
        config.add_option("PHYSICAL-OFFSET", short_option = 'P', 
                          default = False, cache_invalidator = False, 
                          help = "Display physical offsets instead of virtual", 
                          action = "store_true")

    def unified_output(self, data):
        offsettype = "(V)" if not self._config.PHYSICAL_OFFSET else "(P)"
        return TreeGrid([("Offset{0}".format(offsettype), Address),
                       ("Name", str),
                       ("PID", int),
                       ("PPID", int),
                       ("Sess", int),
                       ("JobSess", int),
                       ("Wow64", int),
                       ("Total", int),
                       ("Active", int),
                       ("Term", int),
                       ("Joblink", str),
                       ("Process", str),],
                        self.generator(data))

    def generator(self, data):
        for task in data:
            job = task.Job.dereference()
            if job:
                if not self._config.PHYSICAL_OFFSET:
                    offset = task.obj_offset
                else:
                    offset = task.obj_vm.vtop(task.obj_offset)
                yield (0, [
                    Address(offset),
                    str(task.ImageFileName),
                    int(task.UniqueProcessId),
                    int(task.InheritedFromUniqueProcessId),
                    int(task.SessionId),
                    int(job.SessionId),
                    int(task.IsWow64),
                    int(job.TotalProcesses),
                    int(job.ActiveProcesses),
                    int(job.TotalTerminatedProcesses),
                    "-",
                    "(Original Process)"])

                for item in job.ProcessListHead.list_of_type("_EPROCESS", "JobLinks"):
                    if not self._config.PHYSICAL_OFFSET: 
                        offset = item.obj_offset    
                    else:       
                        offset = item.obj_vm.vtop(item.obj_offset)
                    path = str(item.ImageFileName)
                    if item.Peb:
                        path = str(item.Peb.ProcessParameters.ImagePathName.v().encode("utf8", "ignore"))
                    yield (0, [
                        Address(offset),
                        str(item.ImageFileName),
                        int(item.UniqueProcessId),
                        int(item.InheritedFromUniqueProcessId),
                        int(item.SessionId),
                        0,
                        int(item.IsWow64),
                        0,
                        0,
                        0,
                        "Yes",
                        path])


    def render_text(self, outfd, data):
        header = "*" * 107
        offsettype = "(V)" if not self._config.PHYSICAL_OFFSET else "(P)"
        self.table_header(outfd,
                          [("Offset{0}".format(offsettype), "[addrpad]"),
                           ("Name", "20s"),
                           ("PID", ">6"),
                           ("PPID", ">6"),
                           ("Sess", ">6"),
                           ("JobSess", ">7"),
                           ("Wow64", ">6"),
                           ("Total", ">6"),
                           ("Active", ">6"),
                           ("Term", ">6"),
                           ("JobLink", ">8"),
                           ("Process", "")]
                          ) 
        for task in data:
            job = task.Job.dereference()
            if job:
                if not self._config.PHYSICAL_OFFSET:
                    offset = task.obj_offset
                else:
                    offset = task.obj_vm.vtop(task.obj_offset)
                self.table_row(outfd,
                    offset,
                    task.ImageFileName,
                    task.UniqueProcessId,
                    task.InheritedFromUniqueProcessId,
                    task.SessionId,
                    job.SessionId,
                    task.IsWow64,
                    job.TotalProcesses,
                    job.ActiveProcesses,
                    job.TotalTerminatedProcesses,
                    "-",
                    "(Original Process)")

                for item in job.ProcessListHead.list_of_type("_EPROCESS", "JobLinks"):
                    if not self._config.PHYSICAL_OFFSET: 
                        offset = item.obj_offset                
                    else:                    
                        offset = item.obj_vm.vtop(item.obj_offset)
                    self.table_row(outfd,
                        offset,
                        item.ImageFileName,
                        item.UniqueProcessId,
                        item.InheritedFromUniqueProcessId,
                        item.SessionId,
                        "-",
                        item.IsWow64,
                        "-",
                        "-",
                        "-",
                        "Yes",
                        item.Peb.ProcessParameters.ImagePathName.v().encode("utf8", "ignore"))
                outfd.write("{0}\n".format(header))



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# Additional Authors:
# Michael Cohen <scudette@users.sourceforge.net>
# Mike Auty <mike.auty@gmail.com>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

#pylint: disable-msg=C0111

import os, re
import volatility.plugins.common as common
from volatility.renderers import TreeGrid
from volatility.renderers.basic import Address, Hex
import volatility.win32 as win32
import volatility.obj as obj
import volatility.debug as debug
import volatility.utils as utils
import volatility.cache as cache

class DllList(common.AbstractWindowsCommand, cache.Testable):
    """Print list of loaded dlls for each process"""

    def __init__(self, config, *args, **kwargs):
        common.AbstractWindowsCommand.__init__(self, config, *args, **kwargs)
        cache.Testable.__init__(self)
        config.add_option('OFFSET', short_option = 'o', default = None,
                          help = 'EPROCESS offset (in hex) in the physical address space',
                          action = 'store', type = 'int')

        config.add_option('PID', short_option = 'p', default = None,
                          help = 'Operate on these Process IDs (comma-separated)',
                          action = 'store', type = 'str')

        config.add_option('NAME', short_option = 'n', default = None,
                          help = 'Operate on these process names (regex)',
                          action = 'store', type = 'str')

    def unified_output(self, data):
        return TreeGrid([("Pid", int),
                       ("Base", Address),
                       ("Size", Hex),
                       ("LoadCount", Hex),
                       ("Path", str)],
                        self.generator(data))

    def generator(self, data):
        for task in data:
            pid = task.UniqueProcessId

            if task.Peb:
                for m in task.get_load_modules():
                    yield (0, [int(pid), Address(m.DllBase), Hex(m.SizeOfImage), Hex(m.LoadCount), str(m.FullDllName or '')])
            else:
                yield (0, [int(pid), Address(0), Hex(0), Hex(0), "Error reading PEB for pid"])

    def render_text(self, outfd, data):
        for task in data:
            pid = task.UniqueProcessId

            outfd.write("*" * 72 + "\n")
            outfd.write("{0} pid: {1:6}\n".format(task.ImageFileName, pid))

            if task.Peb:
                ## REMOVE this after 2.4, since we have the cmdline plugin now
                outfd.write("Command line : {0}\n".format(str(task.Peb.ProcessParameters.CommandLine or '')))
                if task.IsWow64:
                    outfd.write("Note: use ldrmodules for listing DLLs in Wow64 processes\n")
                outfd.write("{0}\n".format(str(task.Peb.CSDVersion or '')))
                outfd.write("\n")
                self.table_header(outfd,
                                  [("Base", "[addrpad]"),
                                   ("Size", "[addr]"),
                                   ("LoadCount", "[addr]"),
                                   ("Path", ""),
                                   ])
                for m in task.get_load_modules():
                    self.table_row(outfd, m.DllBase, m.SizeOfImage, m.LoadCount, str(m.FullDllName or ''))
            else:
                outfd.write("Unable to read PEB for task.\n")

    def filter_tasks(self, tasks):
        """ Reduce the tasks based on the user selectable PIDS parameter.

        Returns a reduced list or the full list if config.PIDS not specified.
        """

        if self._config.PID is not None:
            try:
                pidlist = [int(p) for p in self._config.PID.split(',')]
            except ValueError:
                debug.error("Invalid PID {0}".format(self._config.PID))

            pids = [t for t in tasks if t.UniqueProcessId in pidlist]
            if len(pids) == 0:
                debug.error("Cannot find PID {0}. If its terminated or unlinked, use psscan and then supply --offset=OFFSET".format(self._config.PID))
            return pids

        if self._config.NAME is not None:
            try:
                name_re = re.compile(self._config.NAME, re.I)
            except re.error:
                debug.error("Invalid name {0}".format(self._config.NAME))

            names = [t for t in tasks if name_re.search(str(t.ImageFileName))]
            if len(names) == 0:
                debug.error("Cannot find name {0}. If its terminated or unlinked, use psscan and then supply --offset=OFFSET".format(self._config.NAME))
            return names

        return tasks

    @staticmethod
    def virtual_process_from_physical_offset(addr_space, offset):
        """ Returns a virtual process from a physical offset in memory """
        # Since this is a physical offset, we find the process
        flat_addr_space = utils.load_as(addr_space.get_config(), astype = 'physical')
        flateproc = obj.Object("_EPROCESS", offset, flat_addr_space)
        # then use the virtual address of its first thread to get into virtual land
        # (Note: the addr_space and flat_addr_space use the same config, so should have the same profile)
        tleoffset = addr_space.profile.get_obj_offset("_ETHREAD", "ThreadListEntry")

        # start out with the member offset given to us from the profile 
        offsets = [tleoffset]

        # if (and only if) we're dealing with 64-bit Windows 7 SP1 
        # then add the other commonly seen member offset to the list 
        meta = addr_space.profile.metadata
        major = meta.get("major", 0)
        minor = meta.get("minor", 0)
        build = meta.get("build", 0)
        version = (major, minor, build)

        if meta.get("memory_model") == "64bit" and version == (6, 1, 7601):
            offsets.append(tleoffset + 8)

        ## use the member offset from the profile 
        for ofs in offsets:
            ethread = obj.Object("_ETHREAD", offset = flateproc.ThreadListHead.Flink.v() - ofs, vm = addr_space)
            # and ask for the thread's process to get an _EPROCESS with a virtual address space
            virtual_process = ethread.owning_process()
            # Sanity check the bounce. See Issue 154.
            if virtual_process and offset == addr_space.vtop(virtual_process.obj_offset):
                return virtual_process

        return obj.NoneObject("Unable to bounce back from virtual _ETHREAD to virtual _EPROCESS")

    @cache.CacheDecorator(lambda self: "tests/pslist/pid={0}/offset={1}".format(self._config.PID, self._config.OFFSET))
    def calculate(self):
        """Produces a list of processes, or just a single process based on an OFFSET"""
        addr_space = utils.load_as(self._config)

        if self._config.OFFSET != None:
            tasks = [self.virtual_process_from_physical_offset(addr_space, self._config.OFFSET)]
        else:
            tasks = self.filter_tasks(win32.tasks.pslist(addr_space))

        return tasks

class PSList(DllList):
    """ Print all running processes by following the EPROCESS lists """
    def __init__(self, config, *args, **kwargs):
        DllList.__init__(self, config, *args, **kwargs)
        config.add_option("PHYSICAL-OFFSET", short_option = 'P',
                          default = False, cache_invalidator = False,
                          help = "Display physical offsets instead of virtual",
                          action = "store_true")

    def render_text(self, outfd, data):

        offsettype = "(V)" if not self._config.PHYSICAL_OFFSET else "(P)"
        self.table_header(outfd,
                          [("Offset{0}".format(offsettype), "[addrpad]"),
                           ("Name", "20s"),
                           ("PID", ">6"),
                           ("PPID", ">6"),
                           ("Thds", ">6"),
                           ("Hnds", ">8"),
                           ("Sess", ">6"),
                           ("Wow64", ">6"),
                           ("Start", "30"),
                           ("Exit", "30")]
                          )

        for task in data:
            # PHYSICAL_OFFSET must STRICTLY only be used in the results.  If it's used for anything else,
            # it needs to have cache_invalidator set to True in the options
            if not self._config.PHYSICAL_OFFSET:
                offset = task.obj_offset
            else:
                offset = task.obj_vm.vtop(task.obj_offset)
            self.table_row(outfd,
                offset,
                task.ImageFileName,
                task.UniqueProcessId,
                task.InheritedFromUniqueProcessId,
                task.ActiveThreads,
                task.ObjectTable.HandleCount,
                task.SessionId,
                task.IsWow64,
                str(task.CreateTime or ''),
                str(task.ExitTime or ''),
                )

    def render_dot(self, outfd, data):
        objects = set()
        links = set()
 
        for eprocess in data:
            label = "{0} | {1} |".format(eprocess.UniqueProcessId,
                eprocess.ImageFileName)
            if eprocess.ExitTime:
                label += "exited\\n{0}".format(eprocess.ExitTime)
                options = ' style = "filled" fillcolor = "lightgray" '
            else:
                label += "running"
                options = ''
 
            objects.add('pid{0} [label="{1}" shape="record" {2}];\n'.format(eprocess.UniqueProcessId,
                label, options))
            links.add("pid{0} -> pid{1} [];\n".format(eprocess.InheritedFromUniqueProcessId,
                eprocess.UniqueProcessId))
 
        ## Now write the dot file
        outfd.write("digraph processtree { \ngraph [rankdir = \"TB\"];\n")
        for link in links:
            outfd.write(link)

        for item in objects:
            outfd.write(item)
        outfd.write("}")

    def unified_output(self, data):

        offsettype = "(V)" if not self._config.PHYSICAL_OFFSET else "(P)"
        return TreeGrid([("Offset{0}".format(offsettype), Address),
                        ("Name", str),
                        ("PID", int),
                        ("PPID", int),
                        ("Thds", int),
                        ("Hnds", int),
                        ("Sess", int),
                        ("Wow64", int),
                        ("Start", str),
                        ("Exit", str)],
                        self.generator(data))

    def generator(self, data):
        for task in data:
            # PHYSICAL_OFFSET must STRICTLY only be used in the results.  If it's used for anything else,
            # it needs to have cache_invalidator set to True in the options
            if not self._config.PHYSICAL_OFFSET:
                offset = task.obj_offset
            else:
                offset = task.obj_vm.vtop(task.obj_offset)
            yield (0, [Address(offset),
                             str(task.ImageFileName),
                             int(task.UniqueProcessId),
                             int(task.InheritedFromUniqueProcessId),
                             int(task.ActiveThreads),
                             int(task.ObjectTable.HandleCount),
                             int(task.SessionId),
                             int(task.IsWow64),
                             str(task.CreateTime or ''),
                             str(task.ExitTime or '')])

# Inherit from files just for the config options (__init__)
class MemMap(DllList):
    """Print the memory map"""

    def unified_output(self, data):
        return TreeGrid([("Process", str),
                       ("PID", int),
                       ("Virtual", Address),
                       ("Physical", Address),
                       ("Size", Address),
                       ("DumpFileOffset", Address)],
                        self.generator(data))

    def generator(self, data):
        for pid, task, pagedata in data:
            task_space = task.get_process_address_space()
            proc = "{0}".format(task.ImageFileName)
            offset = 0
            if pagedata:
                for p in pagedata:
                    pa = task_space.vtop(p[0])
                    # pa can be 0, according to the old memmap, but can't == None(NoneObject)
                    if pa != None:
                        data = task_space.read(p[0], p[1])
                        if data != None:
                            yield (0, [proc, int(pid), Address(p[0]), Address(pa), Address(p[1]), Address(offset)])
                            offset += p[1]

    def render_text(self, outfd, data):
        first = True
        for pid, task, pagedata in data:
            if not first:
                outfd.write("*" * 72 + "\n")

            task_space = task.get_process_address_space()
            outfd.write("{0} pid: {1:6}\n".format(task.ImageFileName, pid))
            first = False

            offset = 0
            if pagedata:
                self.table_header(outfd,
                                  [("Virtual", "[addrpad]"),
                                   ("Physical", "[addrpad]"),
                                   ("Size", "[addr]"),
                                   ("DumpFileOffset", "[addr]")])

                for p in pagedata:
                    pa = task_space.vtop(p[0])
                    # pa can be 0, according to the old memmap, but can't == None(NoneObject)
                    if pa != None:
                        data = task_space.read(p[0], p[1])
                        if data != None:
                            self.table_row(outfd, p[0], pa, p[1], offset)
                            offset += p[1]
            else:
                outfd.write("Unable to read pages for task.\n")

    @cache.CacheDecorator(lambda self: "tests/memmap/pid={0}/offset={1}".format(self._config.PID, self._config.OFFSET))
    def calculate(self):
        tasks = DllList.calculate(self)

        for task in tasks:
            if task.UniqueProcessId:
                pid = task.UniqueProcessId
                task_space = task.get_process_address_space()
                pages = task_space.get_available_pages()
                yield pid, task, pages

class MemDump(MemMap):
    """Dump the addressable memory for a process"""

    def __init__(self, config, *args, **kwargs):
        MemMap.__init__(self, config, *args, **kwargs)
        config.add_option('DUMP-DIR', short_option = 'D', default = None,
                          cache_invalidator = False,
                          help = 'Directory in which to dump memory')

    def render_text(self, outfd, data):
        if self._config.DUMP_DIR == None:
            debug.error("Please specify a dump directory (--dump-dir)")
        if not os.path.isdir(self._config.DUMP_DIR):
            debug.error(self._config.DUMP_DIR + " is not a directory")

        for pid, task, pagedata in data:
            outfd.write("*" * 72 + "\n")

            task_space = task.get_process_address_space()
            outfd.write("Writing {0} [{1:6}] to {2}.dmp\n".format(task.ImageFileName, pid, str(pid)))

            f = open(os.path.join(self._config.DUMP_DIR, str(pid) + ".dmp"), 'wb')
            if pagedata:
                for p in pagedata:
                    data = task_space.read(p[0], p[1])
                    if data == None:
                        if self._config.verbose:
                            outfd.write("Memory Not Accessible: Virtual Address: 0x{0:x} Size: 0x{1:x}\n".format(p[0], p[1]))
                    else:
                        f.write(data)
            else:
                outfd.write("Unable to read pages for task.\n")
            f.close()



# Volatility
# Copyright (C) 2007-2015 Volatility Foundation
# Copyright (c) 2015 Michael Ligh <michael.ligh@mnin.org>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import struct
import volatility.utils as utils
import volatility.obj as obj
import volatility.win32.tasks as tasks
import volatility.debug as debug
import volatility.plugins.malware.svcscan as svcscan
import volatility.win32.rawreg as rawreg
import volatility.plugins.registry.hivelist as hivelist

class ServiceDiff(svcscan.SvcScan):
    "List Windows services (ala Plugx)"

    @staticmethod
    def services_from_registry(addr_space):
        """Enumerate services from the cached registry hive"""

        services = {}
        plugin = hivelist.HiveList(addr_space.get_config())
        for hive in plugin.calculate():

            ## find the SYSTEM hive 
            name = hive.get_name()
            if not name.lower().endswith("system"):
                continue 
        
            ## get the root key 
            hive_space = hive.address_space() 
            root = rawreg.get_root(hive_space)

            if not root:
                break 

            ## open the services key 
            key = rawreg.open_key(root, ["ControlSet001", "Services"])
            if not key:
                break 

            ## build a dictionary of the key names 
            for subkey in rawreg.subkeys(key):
                services[(str(subkey.Name).lower())] = subkey

            ## we don't need to keep trying 
            break

        return services

    @staticmethod
    def services_from_memory_list(addr_space):
        """Enumerate services from walking the SCM's linked list"""

        services = {}
        pre_vista = addr_space.profile.metadata.get('major', 0) < 6
        mem_model = addr_space.profile.metadata.get('memory_model', '32bit') 

        if mem_model != "32bit":
            return {}

        ## find the service control manager process 
        for process in tasks.pslist(addr_space):
            if str(process.ImageFileName) != "services.exe":
                continue 

            ## create a DOS header at the process' image base address
            process_space = process.get_process_address_space()
            image_base = process.Peb.ImageBaseAddress
            dos_header = obj.Object("_IMAGE_DOS_HEADER", 
                                    offset = image_base, 
                                    vm = process_space)

            if not dos_header:
                break

            ## the first section (.text) contains the values we need 
            try:
                sections = list(dos_header.get_nt_header().get_sections())
                text_seg = sections[0]
            except ValueError:
                ## couldn't parse the PE header 
                break 
            except IndexError:
                ## no sections were found in the array 
                break 

            ## acquire the text section's data 
            virtual_address = text_seg.VirtualAddress + image_base
            data = process_space.zread(virtual_address, text_seg.Misc.VirtualSize)
            list_head = None

            ## look for the ScInitDatabase signature 
            for offset in utils.iterfind(data, "\xA3"):

                if not (data[offset + 5] == "\xA3" and 
                            data[offset + 10] == "\xA3" and 
                            data[offset + 15] == "\xA3" and 
                            data[offset + 20] == "\xA3" and 
                            data[offset + 25] == "\xE8"): 
                        continue

                ## the beginning of the service database list 
                list_head = obj.Object("unsigned long", 
                                offset = virtual_address + offset + 21, 
                                vm = process_space)

            ## unable to find the signature...means list walking won't work 
            if not list_head:
                break

            record = obj.Object("_SERVICE_RECORD", 
                                offset = list_head, 
                                vm = process_space)

            while record:
                name = str(record.ServiceName.dereference() or '')
                name = name.lower()
                services[name] = record
                record = record.ServiceList.Flink.dereference()

        return services

    @staticmethod
    def compare(reg_list, mem_list):
        """Compare the services found in the registry with those in memory"""

        ## the names of all services in only the registry list 
        missing = set(reg_list.keys()) - set(mem_list.keys())

        for service in missing:
            ## the SCM only loads services with an ImagePath value so make 
            ## sure to skip those entries, as they will not end up in memory 
            has_imagepath = False
            for value in rawreg.values(reg_list[service]):
                if str(value.Name) == "ImagePath":
                    has_imagepath = True
                    break 

            if has_imagepath:
                yield reg_list[service] 

    def calculate(self):
        addr_space = utils.load_as(self._config)

        from_registry = ServiceDiff.services_from_registry(addr_space)
        from_memory = ServiceDiff.services_from_memory_list(addr_space)

        if not from_registry:
            debug.error("Could not enumerate services from the registry")

        if not from_memory:
            debug.error("Could not enumerate services from memory")

        return ServiceDiff.compare(from_registry, from_memory)

    def render_text(self, outfd, data):
        for subkey in data:
            outfd.write("\n{0:<20}: {1}\n".format("Missing service", subkey.Name))
            for value in rawreg.values(subkey):
                value_type, value_data = rawreg.value_data(value)
                outfd.write("{0:<20}: ({1}) {2}\n".format(value.Name, value_type, value_data))



from volatility import debug
from volatility.renderers.basic import Renderer

__author__ = 'mike'

class DotRenderer(Renderer):
    def __init__(self, renderers_func, config):
        self._config = config
        self._columns = None
        self._text_cell_renderers_func = renderers_func
        self._text_cell_renderers = None

    def description(self, node):
        output = []
        for column in self._columns:
            text = self._text_cell_renderers[column.index].render(node.values[column.index])
            output.append((column.name + ": " + text).replace("|", "_").replace("\"", "_"))
        return "|".join(output)

    def _add_node(self, node, data):
        outfd, accumulator = data
        accumulator[node] = max(accumulator.values()) + 1
        outfd.write("  Node" + str(accumulator[node]) + " [label=\"{" + self.description(node) + "}\"];\n")
        if accumulator[node.parent] != 0:
            outfd.write("  Node" + str(accumulator[node.parent]) + " -> Node" + str(accumulator[node]) + ";\n")
        return (outfd, accumulator)

    def render(self, outfd, grid):
        """Renders the TreeGrid in data out to the output file from the config options"""
        self._columns = grid.columns
        self._text_cell_renderers = self._text_cell_renderers_func(self._columns)

        if grid.max_depth() <= 1:
            debug.warning("Dot output will be unhelpful since the TreeGrid is a flat list")
        outfd.write("digraph output {\n  node[shape = Mrecord];\n  # rankdir=LR;\n")
        grid.visit(None, self._add_node, (outfd, {None: 0}))
        outfd.write("}\n")



import json
import sys
import os

"""
Author: Gleeda <jamie.levy@gmail.com>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version
2 of the License, or (at your option) any later version.

parsesummary.py [summary_file]
    Parses summary files created from the dumpfiles plugin

"""

def usage(name):
    print "{0} [summary_file]".format(name)

def main():
    try:
        summary = sys.argv[1]
        if os.path.isfile(summary):
            f = open(summary, "r")
        else:
            print summary, "is not a file!"
            usage(sys.argv[0])
            return
    except:
        usage(sys.argv[0])
        return

    heading = "*" * 80
    for line in f.readlines():
        print heading
        item = json.loads(line.strip())
        print "File: {0} -> {1}".format(item["name"], item["ofpath"])
        print "\tPID: {0}".format(item["pid"])
        print "\t_FILE_OBJECT offset: 0x{0:x}".format(item["fobj"])
        print "\tType: {0}".format(item["type"])
        vacbary = item.get("vacbary", [])
        if item["type"] == "SharedCacheMap" and vacbary != []:
            for vacb in vacbary:
                print "\tSize: {0}".format(vacb["size"])
                present = vacb.get("present", None)
                padding = vacb.get("pad", None)
                if present != None:
                    print "\tPresent Pages:" 
                    for page in present:
                        print "\t\tOffset(V): 0x{0:x}, Length: {1}".format(page[0], page[1])
            
        else:
            present = item.get("present", None)
            if present != None:
                print "\tPresent Pages:"
                if item["type"] != "SharedCacheMap":
                    for page in present:
                        print "\t\tOffset(P) 0x{0:x} FileOffset: 0x{1:x}, Size: {2}".format(page[0], page[1], page[2])
            padding = item.get("pad", None)
        if padding != None:
            print "\tPadding:"
            for pad in padding:
                print "\t\tFileOffset: 0x{0:x} x 0x{1:x}".format(pad[0], pad[1])
    print heading

if __name__ == "__main__":
    main()



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Cem Gurkok
@license:      GNU General Public License 2.0
@contact:      cemgurkok@gmail.com
@organization:
"""
import volatility.obj as obj
import volatility.plugins.mac.common as common
import volatility.debug as debug
import volatility.utils as utils
import volatility.plugins.mac.pstasks as mac_tasks
from volatility.renderers import TreeGrid
from volatility.renderers.basic import Address

## http://hte.sourceforge.net/doxygenized-0.8.0pre1/machostruc_8h-source.html
## documentation for thread state, registry, launch cmd etc

thread_overlay = {
                     "thread": [ None, {
                                            "options": [None, ['Flags', {'target': 'int', 'bitmap': {    
                                                                                                          "TH_OPT_INTMASK": 0,# interrupt / abort level 
                                                                                                          "TH_OPT_INTMASK": 1,# interrupt / abort level 
                                                                                                          "TH_OPT_VMPRIV": 2, # may allocate reserved memory
                                                                                                          "TH_OPT_DTRACE": 3, # executing under dtrace_probe
                                                                                                          "TH_OPT_SYSTEM_CRITICAL": 4, # Thread must always be allowed to run, even under heavy load 
                                                                                                          "TH_OPT_PROC_CPULIMIT": 5, # Thread has a task-wide CPU limit applied to it 
                                                                                                          "TH_OPT_PRVT_CPULIMIT": 6 # Thread has a thread-private CPU limit applied to it
                                                                       }}]],
                                            "state":   [None, ['Flags', {'target': 'int', 'bitmap': {
                                                                                                          "TH_WAIT": 0,
                                                                                                          "TH_SUSP": 1,
                                                                                                          "TH_RUN": 2,
                                                                                                          "TH_UNINT": 3,
                                                                                                          "TH_TERMINATE": 4,
                                                                                                          "TH_TERMINATE2": 5,
                                                                                                          "TH_IDLE": 6, # kAppleProfileTriggerClientThreadModeIdle
                                                                                                          "TH_IDLE_N": 6 << 16 # kAppleProfileTriggerClientThreadModeNotIdle, !TH_IDLE
                                                                       }}]],
                                            "sched_mode": [None, ['Flags', {'target': 'int', 'bitmap': {
                                                                                                          "TH_MODE_REALTIME": 0,  # /* time constraints supplied */
                                                                                                          "TH_MODE_TIMESHARE": 1, # /* use timesharing algorithm */
                                                                                                          "TH_MODE_FAILSAFE": 2,  # /* fail-safe has tripped */
                                                                                                          "TH_MODE_PROMOTED": 3,  # /* sched pri has been promoted */
                                                                                                          "TH_MODE_ABORT": 4,     # /* abort interruptible waits */
                                                                                                          "TH_MODE_ABORTSAFELY": 5, # /* ... but only those at safe point */
                                                                                                          # "TH_MODE_ISABORTED": (TH_MODE_ABORT | TH_MODE_ABORTSAFELY) 
                                                                                                          "TH_MODE_DEPRESS": 6,   # /* normal depress yield */
                                                                                                          "TH_MODE_POLLDEPRESS": 7, # /* polled depress yield */
                                                                                                          # "TH_MODE_ISDEPRESSED": (TH_MODE_DEPRESS | TH_MODE_POLLDEPRESS)
                                                                       }}]],
                                            "ast": [None, ['Flags', {'target': 'int', 'bitmap': { # Asynchronous System Traps
                                                                                                          # AST_NONE , no bits set
                                                                                                          "AST_HALT": 0,
                                                                                                          "AST_TERMINATE": 1,
                                                                                                          "AST_BLOCK": 2,
                                                                                                          "AST_UNUSED": 3,
                                                                                                          "AST_QUANTUM": 4,
                                                                                                          "AST_APC": 5, # /* migration APC hook */
                                                                                                          "AST_URGENT": 6
                                                                       }}]],

                     }]
                 }

class queue_entry(obj.CType):
    # needed a separate walk_list function for threads since the original was task specific
    def thread_walk_list(self, list_head):
        n = self.next.dereference_as("thread")
        while n and n.obj_offset != list_head:
            yield n
            n = n.task_threads.next.dereference_as("thread")
        p = self.prev.dereference_as("thread")
        while p and p.obj_offset != list_head:
            yield p
            p = p.task_threads.prev.dereference_as("thread")

    def walk_list(self, list_head):
        n = self.next.dereference_as("task")
        while n and n.obj_offset != list_head:
            yield n
            n = n.tasks.next.dereference_as("task")
        p = self.prev.dereference_as("task")
        while p and p.obj_offset != list_head:
            yield p
            p = p.tasks.prev.dereference_as("task")

class MacObjectClasses2(obj.ProfileModification):

    conditions = {'os': lambda x: x == 'mac'}
    before = ['BasicObjectClasses']

    def modification(self, profile):
        profile.object_classes.update({
            'queue_entry' : queue_entry
        })

class MacObjectClasses4(obj.ProfileModification):

    conditions = {'os': lambda x: x == 'mac'}
    before = ['BasicObjectClasses']

    def modification(self, profile):
        profile.merge_overlay(thread_overlay)

# https://www.opensource.apple.com/source/xnu/xnu-124.1/osfmk/mach/vm_statistics.h
dict_alias = {
    1: "VM_MEMORY_MALLOC",
    2: "VM_MEMORY_MALLOC_SMALL",
    3: "VM_MEMORY_MALLOC_LARGE",
    4: "VM_MEMORY_MALLOC_HUGE",
    5: "VM_MEMORY_SBRK",
    6: "VM_MEMORY_REALLOC",
    7: "VM_MEMORY_MALLOC_TINY",
    8: "VM_MEMORY_MALLOC_LARGE_REUSABLE",
    9: "VM_MEMORY_MALLOC_LARGE_REUSED",
    10: "VM_MEMORY_ANALYSIS_TOOL",
    20: "VM_MEMORY_MACH_MSG",
    21: "VM_MEMORY_IOKIT",
    30: "VM_MEMORY_STACK",
    31: "VM_MEMORY_GUARD",
    32: "VM_MEMORY_SHARED_PMAP",
    33: "VM_MEMORY_DYLIB",
    34: "VM_MEMORY_OBJC_DISPATCHERS",
    35: "VM_MEMORY_UNSHARED_PMAP",
    40: "VM_MEMORY_APPKIT",
    41: "VM_MEMORY_FOUNDATION",
    42: "VM_MEMORY_COREGRAPHICS",
    43: "VM_MEMORY_CORESERVICES",
    44: "VM_MEMORY_JAVA",
    50: "VM_MEMORY_ATS",
    51: "VM_MEMORY_LAYERKIT",
    52: "VM_MEMORY_CGIMAGE",
    53: "VM_MEMORY_TCMALLOC",
    54: "VM_MEMORY_COREGRAPHICS_DATA",
    55: "VM_MEMORY_COREGRAPHICS_SHARED",
    56: "VM_MEMORY_COREGRAPHICS_FRAMEBUFFERS",
    57: "VM_MEMORY_COREGRAPHICS_BACKINGSTORES",
    60: "VM_MEMORY_DYLD",
    61: "VM_MEMORY_DYLD_MALLOC",
    62: "VM_MEMORY_SQLITE",
    63: "VM_MEMORY_JAVASCRIPT_CORE",
    64: "VM_MEMORY_JAVASCRIPT_JIT_EXECUTABLE_ALLOCATOR",
    65: "VM_MEMORY_JAVASCRIPT_JIT_REGISTER_FILE",
    66: "VM_MEMORY_GLSL",
    67: "VM_MEMORY_OPENCL",
    68: "VM_MEMORY_COREIMAGE",
    69: "VM_MEMORY_WEBCORE_PURGEABLE_BUFFERS",
    70: "VM_MEMORY_IMAGEIO",
    71: "VM_MEMORY_COREPROFILE",
    72: "VM_MEMORY_ASSETSD",
    240: "VM_MEMORY_APPLICATION_SPECIFIC_1",
    241: "VM_MEMORY_APPLICATION_SPECIFIC_2",
    242: "VM_MEMORY_APPLICATION_SPECIFIC_3",
    243: "VM_MEMORY_APPLICATION_SPECIFIC_4",
    244: "VM_MEMORY_APPLICATION_SPECIFIC_5",
    245: "VM_MEMORY_APPLICATION_SPECIFIC_6",
    246: "VM_MEMORY_APPLICATION_SPECIFIC_7",
    247: "VM_MEMORY_APPLICATION_SPECIFIC_8",
    248: "VM_MEMORY_APPLICATION_SPECIFIC_9",
    249: "VM_MEMORY_APPLICATION_SPECIFIC_10",
    250: "VM_MEMORY_APPLICATION_SPECIFIC_11",
    251: "VM_MEMORY_APPLICATION_SPECIFIC_12",
    252: "VM_MEMORY_APPLICATION_SPECIFIC_13",
    253: "VM_MEMORY_APPLICATION_SPECIFIC_14",
    254: "VM_MEMORY_APPLICATION_SPECIFIC_15",
    255: "VM_MEMORY_APPLICATION_SPECIFIC_16"
}



class mac_threads(mac_tasks.mac_tasks):
    """ List Process Threads """

    def get_active_threads(self):
        threads = {}
        real_ncpus = obj.Object("int", offset = self.addr_space.profile.get_symbol("_real_ncpus"), vm = self.addr_space)
        cpu_data_ptrs = obj.Object(theType = 'Array', offset = self.addr_space.profile.get_symbol("_cpu_data_ptr"), vm = self.addr_space, targetType = "unsigned long long", count = real_ncpus)
        for i in range(0, real_ncpus):
            cpu_data = obj.Object('cpu_data', offset = cpu_data_ptrs[i], vm = self.addr_space)
            threads[i] = cpu_data.cpu_active_thread

        return threads

    def is_thread_active(self, thread, active_threads):
        for active_thread in active_threads.values():
            if active_thread.v() == thread.v():
                return True
        return False

    def get_stack_map(self, proc, proc_threads, bit_string):
        proc_addrspace = proc.get_process_address_space()
        vm_map_slide = 0
        vm_map_start = 0
        stack_thread_id = None
        maps = []

        for map in proc.get_proc_maps():
            vm_map_start = map.links.start
            map_type = str(dict_alias.get(int(map.alias), "UNKNOWN"))
            map_path = map.get_path()

            # see if map is a STACK (not a STACK_GUARD), if so which thread it belongs to

            # VM_MEMORY_STACK
            if map_type == "VM_MEMORY_STACK" and map.get_perms() != "---":

                for thread in proc_threads:
                    # 64bit thread
                    if "64" in bit_string:
                        # isf: interrupt stack frame
                        thread_sp = thread.machine.iss.uss.ss_64.isf.rsp
                    # 32 bit thread
                    else:
                        thread_sp = thread.machine.iss.uss.ss_32.uesp

                    if map.links.start <= thread_sp and thread_sp <= map.links.end:
                        stack_thread_id = thread.thread_id
                        map_path = "thread id {0}".format(thread.thread_id)
                        break

                if "thread" in map_path:
                # Based on the vmmap command:
                 # current map is a stack marked as thread, then mark previous map with thread id if stack
                    prev_proc, prev_map, prev_map_path = maps.pop()
                    if str(dict_alias.get(int(prev_map.alias), "UNKNOWN")) == "VM_MEMORY_STACK" and prev_map.get_perms() != "---" and "thread" not in prev_map_path:
                        prev_map_path = "thread id {0}".format(stack_thread_id)
                    maps.append((prev_proc, prev_map, prev_map_path))
                else:
                 # if previous map is a stack marked as thread, then mark current map with thread id
                    prev_proc, prev_map, prev_map_path = maps.pop()
                    if str(dict_alias.get(int(prev_map.alias), "UNKNOWN")) == "VM_MEMORY_STACK" and prev_map.get_perms() != "---" and "thread" in prev_map_path:
                        map_path = "thread id {0}".format(stack_thread_id)
                    maps.append((prev_proc, prev_map, prev_map_path))

            elif map_type != "VM_MEMORY_STACK":
                stack_thread_id = None

            maps.append((proc, map, map_path))

        return maps

    def get_thread_registers(self, thread, bit_string):
        # http://www.opensource.apple.com/source/xnu/xnu-2050.18.24/osfmk/i386/pcb.c
        registers_64 = ['rdi','rsi','rdx','rbp','rbx','rcx','rax','cr2','r8','r9','r10','r11','r12','r13','r14','r15','gs','fs']
        registers_32 = ['edi','esi','edx','ebp','ebx','uesp','ecx','eax','eip','cr2','gs','cs','fs','es','ds']
        registers = {}

        if "64" in bit_string:
            registers['rsp'] = "{0:#10x}".format(getattr(thread.machine.iss.uss.ss_64.isf, 'rsp'))
            registers['rip'] = "{0:#10x}".format(getattr(thread.machine.iss.uss.ss_64.isf, 'rip'))
            registers['ss'] = "{0:#10x}".format(getattr(thread.machine.iss.uss.ss_64.isf, 'ss'))
            registers['trapno'] = "{0:#10x}".format(getattr(thread.machine.iss.uss.ss_64.isf, 'trapno'))

            # check if trap function/sysent is known or hooked
            trapfn_addr = getattr(thread.machine.iss.uss.ss_64.isf, 'trapfn')

            if trapfn_addr == 0:
                trapfn_name = ''
            else:
                trapfn_name = self.addr_space.profile.get_symbol_by_address('kernel', trapfn_addr)

            if trapfn_name == '':
                trapfn = "UNKNOWN function at {0}".format(trapfn_addr) 
            else:
                trapfn = "{0} at {1:#10x}".format(trapfn_name, trapfn_addr)

            registers['trapfn'] = trapfn

            for reg in registers_64:
                registers[reg] = "{0:#10x}".format(getattr(thread.machine.iss.uss.ss_64, reg))
        else:
            for reg in registers_32:
                if hasattr(thread.machine, "iss"):
                    registers[reg] = "{0:#10x}".format(getattr(thread.machine.iss.uss.ss_32, reg))
                else:
                    registers[reg] = ""

        return registers

    def calculate(self):
        common.set_plugin_members(self)

        for proc in mac_tasks.mac_tasks(self._config).calculate():
            bit_string = str(proc.task.map.pmap.pm_task_map or '')[9:]

            # get proc args and arg address
            args = proc.get_arguments()
            args_addr = proc.user_stack - proc.p_argslen

            # get threads
            qentry = proc.task.threads
            seen_threads = []
            thread_list = []
            active_threads = self.get_active_threads()

            for thread in qentry.thread_walk_list(qentry.obj_offset):
                if thread.obj_offset not in seen_threads:
                    seen_threads.append(thread.obj_offset)
                    thread_list.append(thread)

            # get proc maps
            maps = self.get_stack_map(proc, thread_list, bit_string)
            # get thread stack start and size
            for thread in thread_list:
                stack_start  = 0
                stack_size   = 0
                thread_args  = ""
                registers    = {}
                is_active    = "NO"
                dtraced      = "NO"
                debugged     = "NO"
                uid          = "NONE"

                for proc, map, map_path in maps:
                    if "thread id {0}".format(thread.thread_id) in map_path:
                        if stack_start == 0 or stack_start > map.links.start:
                            stack_start = map.links.start
                        stack_size += map.links.end - map.links.start
                        # find thread with args, which probably is main thread
                        if map.links.start < args_addr < map.links.end:
                            thread_args = args

                # kernel_stack process
                # thread stack information is empty for kernel threads
                if str(proc.p_pid) == "0":
                    stack_start = thread.kernel_stack

                registers = self.get_thread_registers(thread, bit_string)
                if self.is_thread_active(thread, active_threads):
                    is_active = "YES"

                # check if thread is being hardware debugged, ids = x86_debug_state64
                if thread.machine.ids != 0:
                    debugged = "YES"

                # check if dtrace probe is applied
                if "TH_OPT_DTRACE" in str(thread.options):
                    dtraced = "YES"

                #get thread User ID
                #if thread.uthread != 0:
                #uid = thread.uthread.dereference_as('uthread').uu_context.vc_ucred.cr_posix.cr_uid

                yield proc, thread, stack_start, stack_size, thread_args, registers, is_active, dtraced, debugged, uid

            proc = proc.p_list.le_next.dereference()
        
        self.get_active_threads()

    def unified_output(self, data):
        return TreeGrid([("Offset", Address),
                                  ("Pid", int),
                                  ("Tid", int),
                                  ("UID", str),
                                  ("State", str),
                                  ("Is Active?", str),
                                  ("Options", str),
                                  ("Priority", int),
                                  ("Startup Addr", Address),
                                  ("Stack Start Addr", Address),
                                  ("Stack Size (bytes)", int),
                                  ("HW Debugged",str),
                                  ("DTraced", str),
                                  ("Arguments", str),
                                  ], self.generator(data))
                          
    def generator(self, data):
        for proc, thread, stack_start, stack_size, args, registers, is_active, dtraced, debugged, uid in data:
            if not thread.is_valid():
                continue

            yield (0, [
                        Address(thread.v()),
                        int(proc.p_pid),
                        int(thread.thread_id),
                        str(uid),
                        str(thread.state),
                        str(is_active),
                        str(thread.options),
                        int(thread.sched_pri),
                        Address(thread.continuation),
                        Address(stack_start),
                        int(stack_size),
                        str(debugged),
                        str(dtraced),
                        str(args),
                        ])

    def render_text(self, outfd, data):
        self.table_header(outfd, [("Offset", "[addrpad]"),
                                  ("Pid", "8"),
                                  ("Tid", "8"),
                                  ("UID", "8"),
                                  ("State", "30"),
                                  ("Is Active?","<10"),
                                  ("Options", "30"),
                                  ("Priority", "8"),
                                  ("Startup Addr", "[addrpad]"),
                                  ("Stack Start Addr", "[addrpad]"),
                                  ("Stack Size (bytes)", "<18"),
                                  ("HW Debugged","<11"),
                                  ("DTraced","<7"),
                                  ("Arguments", "")
                          ])

        for proc, thread, stack_start, stack_size, args, registers, is_active, dtraced, debugged, uid in data:
            if not thread.is_valid():
                continue

            self.table_row(outfd, thread.v(),
                                  str(proc.p_pid),
                                  str(thread.thread_id),
                                  str(uid),
                                  str(thread.state),
                                  is_active,
                                  str(thread.options),
                                  str(thread.sched_pri),
                                  thread.continuation,
                                  stack_start,
                                  stack_size,
                                  debugged,
                                  dtraced,
                                  args
                           )
            #for reg in registers:
            #    outfd.write("\t{0:<10} {1:}\n".format(reg, registers[reg].strip()))



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import volatility.obj as obj
import volatility.plugins.linux.common as linux_common
import volatility.plugins.linux.pslist as linux_pslist

class linux_getcwd(linux_pslist.linux_pslist):
    """Lists current working directory of each process"""

    def render_text(self, outfd, data):

        self.table_header(outfd, [("Name", "17"),
                                  ("Pid", "8"),
                                  ("CWD", "")])

        for task in data:
            self.table_row(outfd, str(task.comm), task.pid, task.getcwd())



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import volatility.obj as obj
import volatility.plugins.linux.common as linux_common
import volatility.plugins.linux.pslist as linux_pslist
import volatility.plugins.linux.proc_maps as linux_proc_maps

class linux_proc_maps_rb(linux_proc_maps.linux_proc_maps):
    """Gathers process maps for linux through the mappings red-black tree"""

    def calculate(self):
        linux_common.set_plugin_members(self)
        tasks = linux_pslist.linux_pslist.calculate(self)

        for task in tasks:
            if task.mm:
                for vma in task.get_proc_maps_rb():
                    yield task, vma            




# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case and Golden G. Richard III
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com / golden@arcanealloy.com
@organization: 
"""

import os
import volatility.obj as obj
import volatility.debug as debug
import volatility.plugins.mac.common as common
import volatility.plugins.mac.proc_maps as proc_maps
import struct
import WKdm

class mac_dump_maps(proc_maps.mac_proc_maps):
    """ Dumps memory ranges of process(es), optionally including pages in compressed swap """

    def __init__(self, config, *args, **kwargs):         
        proc_maps.mac_proc_maps.__init__(self, config, *args, **kwargs)         
        self._config.add_option('MAP-ADDRESS', short_option = 's', default = None, help = 'Filter by starting address of map', action = 'store', type = 'long') 
        self._config.add_option('OUTPUTFILE', short_option = 'O', default = None, help = 'Output File', action = 'store', type = 'str')
        self._config.add_option('DECOMPRESS-SWAP', default = False, help = 'Also decompress pages in compressed swap', action = 'store_true')
        self._config.add_option('ONLY-DECOMPRESSED-SWAP', default = False, help = 'Dump only successfully decompressed swap pages, nothing else', action = 'store_true')
        self._config.add_option('SKIP-WRITING', short_option = 't',
                                help = 'Skip writing pages, just print stats and optionally test decompression',
                                action = 'store_true', default = False)

        # defined in osfmk/vm/vm_compressor.h; proper decompression relies on these
        self.C_SEG_BUFSIZE =               (1024 * 256)
        self.C_SEG_ALLOCSIZE =             (self.C_SEG_BUFSIZE + 4096)
        self.C_SEG_SLOT_ARRAYS =            6
        self.C_SEG_SLOT_ARRAY_SIZE =       64        
        self.C_SEG_SLOT_ARRAY_MASK =       (self.C_SEG_SLOT_ARRAY_SIZE - 1)
        self.C_SEG_OFFSET_ALIGNMENT_MASK = 0x3

        # defined in osfmk/vm/vm_compressor_pager.c; proper slot lookup relies on these

        self.COMPRESSOR_SLOTS_CHUNK_SIZE = 512
        self.COMPRESSOR_SLOTS_PER_CHUNK  = 128    #   (COMPRESSOR_SLOTS_CHUNK_SIZE / sizeof (compressor_slot_t)), compressor_slot_t is a 32-bit int       

        # WKdm decompression in Python
        self.wkdm=WKdm.WKdm()

        self.dest = [0] * self.wkdm.PAGE_SIZE_IN_WORDS

        self.successful_decompressions = 0

        # don't try to deal with maps larger than this--just skip them
        self.MAXMAPSIZE = 16000000000

    def compressed_page_location(self, outfd, map, addr):
        # return (seg, idx) pair that identifies the location of a
        # compressed page starting at 'addr' and belonging to a
        # vm_map_entry 'map' in the compressor store.  Returns (None,
        # None) if the compressor doesn't own this page.

        # based on compressor_pager_slot_lookup() in osfmk/vm/vm_compressor_pager.c and 
        # c_decompress_page in osfmk/vm_compressor.c

        vm_obj = map.object.vm_object
        if not vm_obj.is_valid() or vm_obj.pager_created == 0 or vm_obj.pager_initialized == 0 or vm_obj.pager_ready == 0:
            # compressor can't own pages from this object--object has no pager or pager isn't initialized
            (seg, idx) = (None, None)
        else:
            #print "PAGING OFFSET: " + str(vm_obj.paging_offset)
            addr += vm_obj.paging_offset
            page_num = addr / self.wkdm.PAGE_SIZE_IN_BYTES
            pager = vm_obj.pager.dereference_as("compressor_pager")
            pager_name = pager.cpgr_pager_ops.memory_object_pager_name.dereference_as("char")
            if pager_name != "c":   # "compressor pager" in pager ops 
                # if the pager isn't the compressor_pager, then move on
                # print "  Corresponding pager " + pager_name + " isn't the compressor pager. Substituting zero page." 
                (seg, idx) = (None, None)
            elif not pager.is_valid():
            # pager isn't initialized 
                outfd.write("  Pager isn't initialized. Substituting zero page.\n")
                (seg, idx) = (None, None)
            # page is out of range
            elif page_num > pager.cpgr_num_slots:
                outfd.write("  page_num > pager.cpgr_num_slots: " + str(page_num) + " " + str(pager.cpgr_num_slots) + ".  Substituting zero page.\n")
                (seg, idx) = (None, None)
            else:
                #print "## " + str(pager.cpgr_num_slots)
                #print "## " + str(self.COMPRESSOR_SLOTS_PER_CHUNK)
                num_chunks = (pager.cpgr_num_slots + self.COMPRESSOR_SLOTS_PER_CHUNK - 1) / self.COMPRESSOR_SLOTS_PER_CHUNK
                if num_chunks > 1:
                    # array of chunks
                    chunk_idx = page_num / self.COMPRESSOR_SLOTS_PER_CHUNK
                    cpgr_islots = obj.Object("Array", offset = pager.cpgr_slots.cpgr_islots, targetType = "Pointer",   
                                            count = num_chunks, vm = self.addr_space)                                  
                    chunks_ptr = cpgr_islots[chunk_idx]
                    if chunks_ptr.is_valid():
                        chunk = obj.Object("Array", offset = chunks_ptr, targetType = "unsigned int",  # compressor_slot_t
                                           count = self.COMPRESSOR_SLOTS_PER_CHUNK, vm = self.addr_space)
                        slot_idx = page_num % self.COMPRESSOR_SLOTS_PER_CHUNK                        

                        # chunk[slot_idx] is actually a c_slot_mapping
                        # struct  c_slot_mapping {
                        #        uint32_t        s_cseg:22,      /* segment number + 1 */
                        #                        s_cindx:10;     /* index in the segment */
                        # };

                        # print "DOUBLE LEVEL SEGIDX bitfield is " + str(chunk[slot_idx])

                        seg = chunk[slot_idx] & 0x3FFFFF
                        idx = chunk[slot_idx] >> 22
                    else:
                        (seg, idx) = (None, None)
                else:
                    slot_idx = page_num;
                    cpgr_dslots = obj.Object("Array", offset = pager.cpgr_slots.cpgr_dslots, targetType = "unsigned int",   # actually compressor_slot_t, == int; 
                                            count = pager.cpgr_num_slots, vm = self.addr_space)                             # unsigned here because we have to 

                    # cpgr_dslots[slot_idx] is actually a c_slot_mapping:
                    # struct  c_slot_mapping {
                    #        uint32_t        s_cseg:22,      /* segment number + 1 */
                    #                        s_cindx:10;     /* index in the segment */
                    # };

                    # print "SINGLE LEVEL SEGIDX bitfield is " + str(cpgr_dslots[slot_idx])
                    
                    seg = cpgr_dslots[slot_idx] & 0x3FFFFF
                    idx = cpgr_dslots[slot_idx] >> 22

        return (seg, idx)                


    def decompress(self, outfd, seg, idx):
        # decompress and return 4K page identified by (seg, idx).  Returns None if decompression fails.
        page = None
        if seg >= self.c_segment_count or seg < 1:
            outfd.write("  Segment out of bounds: " + str(seg) + ". Must be > 0 and < c_segment_count == " + str(self.c_segment_count) + ". Substituting zero page.\n")
        else:
            c_seg = self.c_segments[seg - 1].c_seg          # seg is actually segment index + 1
            if c_seg.c_ondisk == 1:
                outfd.write("  Segment " + str(seg) + " is swapped out. Substituting zero page.\n")
            else:
                j1 = idx / self.C_SEG_SLOT_ARRAY_SIZE
                j2 = idx & self.C_SEG_SLOT_ARRAY_MASK

                cslot_array = c_seg.c_slots[j1]
                if cslot_array.is_valid():
                    cslots = obj.Object("Array", offset = cslot_array, targetType = "c_slot", 
                                        count = self.C_SEG_SLOT_ARRAY_SIZE, vm = self.addr_space)
                    cslot=cslots[j2]
                    (csize, compressed, status) = (4096 / 4, False, "UNCOMPRESSED") if (cslot.c_size == 4095) else (cslot.c_size / 4, True, "COMPRESSED")
                    if csize > 0:
                        outfd.write("  Slot " + str(j1) + ", " + str(j2) + ": offset = " + str(cslot.c_offset * 4) + " bytes, size = " + str(csize * 4) + " bytes, " + status + "\n")
                        page = obj.Object("Array", offset = c_seg.c_store.c_buffer+cslot.c_offset * 4, targetType = "int", 
                                          count = csize, vm = self.addr_space)
                        if compressed:
                            # try to decompress page. Compressed data is fed to WKdm as an array of 32-bit ints.
                            decompressed = self.wkdm.WKdm_decompress(page, self.dest)
                            if decompressed > 0:
                                page = self.dest[:]
                                outfd.write("  Decompression successful.\n")
                            else:
                                outfd.write("  Decompression failed.  Substituting zero page.\n")
                                page = None
                        else:
                            # for uniformity, so len() will work in _read_addr_range()
                            page = page[:]
                            outfd.write("  Decompression successful.\n")

        return page                        

                                
    def render_text(self, outfd, data):
        common.set_plugin_members(self)
        if not self._config.OUTPUTFILE:
            debug.error("Please specify an OUTPUTFILE")
        elif os.path.exists(self._config.OUTPUTFILE):
            debug.error("Cowardly refusing to overwrite an existing file.")
                    
        outfile = open(self._config.OUTPUTFILE, "wb+")
        map_address = self._config.MAP_ADDRESS

        size = 0

        self.table_header(outfd, [("Pid", "8"), 
                          ("Name", "20"),
                          ("Start", "#018x"),
                          ("End", "#018x"),
                          ("Perms", "9"),
                          ("Map Name", "")])

        # from osfmk/vm/vm_object.h.  compressor_object is the high level VM object.
        self.compressor_object = obj.Object("vm_object", 
                                       offset = self.addr_space.profile.get_symbol("_compressor_object_store"), 
                                       vm = self.addr_space)
        
        # from osfmk/vm/vm_compressor.c.  c_segments is an array of c_segu objects, which track and store compressed pages.
        # c_segment_count is current size of c_segments array.
        self.c_segment_count = obj.Object("unsigned int", 
                                          offset = self.addr_space.profile.get_symbol("_c_segment_count"), 
                                          vm = self.addr_space)
        
        self.c_segments_ptr = obj.Object("Pointer", offset = self.addr_space.profile.get_symbol("_c_segments"),
                                         vm = self.addr_space)
        
        self.c_segments = obj.Object("Array", targetType = "c_segu", count = self.c_segment_count, 
                                     offset = self.c_segments_ptr, vm = self.addr_space)
        
        for proc, map in data:
            self.table_row(outfd, 
                           str(proc.p_pid), proc.p_comm, 
                           map.links.start, 
                           map.links.end, 
                           map.get_perms(), 
                           map.get_path())

            if (map.links.end - map.links.start) > self.MAXMAPSIZE:
                outfd.write("Skipping suspiciously large map, smearing is suspected.  Adjust MAXMAPSIZE to override.\n")
                continue
            if not map_address or map_address == map.links.start: 
                for page in self._read_addr_range(outfd, proc, map):
                    if not page is None:
                        size += self.wkdm.PAGE_SIZE_IN_BYTES
                        if not self._config.SKIP_WRITING:
                            for k in range(0, self.wkdm.PAGE_SIZE_IN_WORDS):
                                outfile.write(struct.pack('<i', page[k]))
                    
        outfile.close()
        outfd.write("Wrote {0} bytes.\n".format(size))
        if self._config.DECOMPRESS_SWAP:
            outfd.write("{0} pages were successfully decompressed.\n".format(self.successful_decompressions))


    def _read_addr_range(self, outfd, proc, map):
        # read pages from the address space for a map and optionally decompress pages in compressed swap.
        mapstart = map.links.start
        mapoffset = map.offset
        start = map.links.start
        end = map.links.end

        # set the as with our new dtb so we can read from userland
        proc_as = proc.get_process_address_space()

        while start < end:
            rawpage = proc_as.read(start, self.wkdm.PAGE_SIZE_IN_BYTES)
            # need a page-sized buffer of 32-bit words, because the decompressor
            # expects that 

            if isinstance(rawpage, obj.NoneObject) or rawpage is None:
                # NoneObject encapsulates an error from read() on the
                # address space--couldn't read a page, so fail and
                # just substitute a page of zeros, below
                page = None
            else:
                i=0
                page = []
                while i < self.wkdm.PAGE_SIZE_IN_WORDS:
                    (intval,) = struct.unpack('<i', rawpage[i*4:i*4+4])
                    page.append(intval)
                    i += 1
                    
            if not page is None and self._config.ONLY_DECOMPRESSED_SWAP:
                # only decompressed pages--don't yield pages that are present
                page = None
            elif page is None and not self._config.DECOMPRESS_SWAP and not self._config.ONLY_DECOMPRESSED_SWAP:
                # page isn't present and not decompressing swap, which is the only other
                # potential source of data for the page
                page = [0] * self.wkdm.PAGE_SIZE_IN_WORDS
            elif page is None:
                # see if the page is in compressed swap--if so, decompress; otherwise,
                # return a zero page like as.zread().  The location of the page in
                # compressed swap is represented by a (seg,idx) tuple.  The address
                # must be an offset from the beginning of the map
                (seg, idx) = self.compressed_page_location(outfd, map, start - mapstart + mapoffset)
                if seg is None or seg == 0:
                    # page isn't owned by compressor
                    if self._config.ONLY_DECOMPRESSED_SWAP:
                        # don't yield anything for pages that can't be decompressed if only dumping
                        # decompressed swap
                        page = None
                    else:
                        page = [0] * self.wkdm.PAGE_SIZE_IN_WORDS
                else:
                    outfd.write("Trying to decompress page for address " + hex(start).rstrip("L") + " with segment, idx: " + str(seg) + ", " + str(idx) + "\n")
                    page = self.decompress(outfd, seg, idx)
                    if page is None:
                        # decompression failed
                        if self._config.ONLY_DECOMPRESSED_SWAP:
                            # don't yield anything for pages that can't be decompressed
                            page = None
                        else:
                            page = [0] * self.wkdm.PAGE_SIZE_IN_WORDS
                    else:
                        self.successful_decompressions += 1
            
            if not page is None:
                # ugly, but don't try to pad pages deliberately set to None
                pagelen = len(page)
                if pagelen != self.wkdm.PAGE_SIZE_IN_WORDS:
                    outfd.write("Page is wrong size: " + str(pagelen) + ". Extending to " + str(self.wkdm.PAGE_SIZE_IN_BYTES) + ".\n")
                    page.extend([0] * (self.wkdm.PAGE_SIZE_IN_WORDS - pagelen))

            yield page
            start = start + self.wkdm.PAGE_SIZE_IN_BYTES


